@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,19 +52,19 @@ int main(int, char **);
 void makefilter(STRBUF *);
 FILE *openfilter(void);
 void closefilter(FILE *);
-void completion(char *, char *, char *);
-void idutils(char *, char *);
-void grep(char *);
-void pathlist(char *, char *);
-void parsefile(int, char **, char *, char *, char *, int);
-void printtag(FILE *, char *);
-int search(char *, char *, char *, int);
-int includepath(char *, char *);
-void ffformat(char *, int, char *);
+void completion(const char *, const char *, const char *);
+void idutils(const char *, const char *);
+void grep(const char *);
+void pathlist(const char *, const char *);
+void parsefile(int, char **, const char *, const char *, const char *, int);
+void printtag(FILE *, const char *);
+int search(const char *, const char *, const char *, int);
+int includepath(const char *, const char *);
+void ffformat(char *, int, const char *);
 
 STRBUF *sortfilter;			/* sort filter		*/
 STRBUF *pathfilter;			/* path convert filter	*/
-char *localprefix;			/* local prefix		*/
+const char *localprefix;		/* local prefix		*/
 int aflag;				/* [option]		*/
 int cflag;				/* command		*/
 int fflag;				/* command		*/
@@ -90,17 +90,17 @@ int show_help;
 int show_filter;			/* undocumented command */
 int use_tagfiles;
 int debug;
-char *extra_options;
+const char *extra_options;
 
 static void
-usage()
+usage(void)
 {
 	if (!qflag)
 		fputs(usage_const, stderr);
 	exit(2);
 }
 static void
-help()
+help(void)
 {
 	fputs(usage_const, stdout);
 	fputs(help_const, stdout);
@@ -154,7 +154,7 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	char *av = NULL;
+	const char *av = NULL;
 	int count;
 	int db;
 	int optchar;
@@ -162,7 +162,7 @@ main(argc, argv)
 	char cwd[MAXPATHLEN+1];			/* current directory	*/
 	char root[MAXPATHLEN+1];		/* root of source tree	*/
 	char dbpath[MAXPATHLEN+1];		/* dbpath directory	*/
-	char *gtags;
+	const char *gtags;
 
 	while ((optchar = getopt_long(argc, argv, "ace:ifgGIlnopPqrstTuvx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
@@ -322,8 +322,7 @@ main(argc, argv)
 	 * print dbpath or rootdir.
 	 */
 	if (pflag) {
-		char *dir = (rflag) ? root : dbpath;
-		fprintf(stdout, "%s\n", dir);
+		fprintf(stdout, "%s\n", (rflag) ? root : dbpath);
 		exit(0);
 	}
 	/*
@@ -385,7 +384,7 @@ main(argc, argv)
 	 */
 	{
 		int unique = 0;
-		char *sort;
+		const char *sort;
 
 		/*
 		 * We cannot depend on the command PATH, because global(1)
@@ -569,10 +568,10 @@ openfilter(void)
 }
 void
 closefilter(op)
-FILE *op;
+	FILE *op;
 {
 	if (op != stdout)
-		if (pclose(op) < 0)
+		if (pclose(op) != 0)
 			die("terminated abnormally.");
 }
 /*
@@ -584,11 +583,11 @@ FILE *op;
  */
 void
 completion(dbpath, root, prefix)
-	char *dbpath;
-	char *root;
-	char *prefix;
+	const char *dbpath;
+	const char *root;
+	const char *prefix;
 {
-	char *p;
+	const char *p;
 	int flags = GTOP_KEY;
 	GTOP *gtop;
 	int db;
@@ -615,7 +614,7 @@ completion(dbpath, root, prefix)
 void
 printtag(op, line)
 	FILE *op;
-	char *line;
+	const char *line;		/* virtually const */
 {
 	if (xflag) {
 		fputs(line, op);
@@ -626,7 +625,7 @@ printtag(op, line)
 		/*
 		 * Split tag line.
 		 */
-		n = split(line, 4, &ptable);
+		n = split((char *)line, 4, &ptable);
 
 		if (tflag) {
 			fputs(ptable.part[0].start, op);	/* tag */
@@ -649,15 +648,15 @@ printtag(op, line)
  */
 void
 idutils(pattern, dbpath)
-	char *pattern;
-	char *dbpath;
+	const char *pattern;
+	const char *dbpath;
 {
 	FILE *ip, *op;
 	STRBUF *ib = strbuf_open(0);
 	char edit[IDENTLEN+1];
-	char *line, *p, *path, *lno;
-	int linenum, count, editlen;
-	char *lid;
+	const char *path, *lno, *lid;
+	int linenum, count;
+	char *p, *line;
 
 	lid = usable("lid");
 	if (!lid)
@@ -666,7 +665,6 @@ idutils(pattern, dbpath)
 	 * convert spaces into %FF format.
 	 */
 	ffformat(edit, sizeof(edit), pattern);
-	editlen = strlen(edit);
 	/*
 	 * make lid command line.
 	 */
@@ -755,14 +753,14 @@ idutils(pattern, dbpath)
  */
 void
 grep(pattern)
-	char *pattern;
+	const char *pattern;
 {
 	FILE *op, *fp;
 	STRBUF *ib = strbuf_open(MAXBUFLEN);
-	char *path, *p;
+	const char *path, *p;
 	char edit[IDENTLEN+1];
-	char *buffer;
-	int linenum, count, editlen;
+	const char *buffer;
+	int linenum, count;
 	int flags = 0;
 	regex_t	preg;
 
@@ -770,7 +768,6 @@ grep(pattern)
 	 * convert spaces into %FF format.
 	 */
 	ffformat(edit, sizeof(edit), pattern);
-	editlen = strlen(edit);
 
 	if (!Gflag)
 		flags |= REG_EXTENDED;
@@ -826,11 +823,11 @@ grep(pattern)
  */
 void
 pathlist(dbpath, av)
-	char *dbpath;
-	char *av;
+	const char *dbpath;
+	const char *av;
 {
 	FILE *op;
-	char *path, *p;
+	const char *path, *p;
 	regex_t preg;
 	int count;
 
@@ -898,15 +895,15 @@ void
 parsefile(argc, argv, cwd, root, dbpath, db)
 	int argc;
 	char **argv;
-	char *cwd;
-	char *root;
-	char *dbpath;
+	const char *cwd;
+	const char *root;
+	const char *dbpath;
 	int db;
 {
 	char buf[MAXPATHLEN+1], *path;
-	char *p;
+	const char *p;
 	FILE *ip, *op;
-	char *parser, *av;
+	const char *parser, *av;
 	int count;
 	STRBUF *sb = strbuf_open(0);
 	STRBUF *com = strbuf_open(0);
@@ -1007,12 +1004,12 @@ parsefile(argc, argv, cwd, root, dbpath, db)
  */
 int
 search(pattern, root, dbpath, db)
-	char *pattern;
-	char *root;
-	char *dbpath;
+	const char *pattern;
+	const char *root;
+	const char *dbpath;
 	int db;
 {
-	char *p;
+	const char *p;
 	int count = 0;
 	FILE *op;
 	GTOP *gtop;
@@ -1044,7 +1041,7 @@ search(pattern, root, dbpath, db)
 		flags |= GTOP_BASICREGEX;
 	for (p = gtags_first(gtop, pattern, flags); p; p = gtags_next(gtop)) {
 		if (lflag) {
-			char *q;
+			const char *q;
 			/* locate start point of a path */
 			q = locatestring(p, "./", MATCH_FIRST);
 			if (!locatestring(q, localprefix, MATCH_AT_FIRST))
@@ -1068,10 +1065,10 @@ search(pattern, root, dbpath, db)
  */
 int
 includepath(line, path)
-	char *line;
-	char *path;
+	const char *line;
+	const char *path;
 {
-	char *p;
+	const char *p;
 	int length;
 
 	if (!(p = locatestring(line, "./", MATCH_FIRST)))
@@ -1095,9 +1092,10 @@ void
 ffformat(to, size, from)
 	char *to;
 	int size;
-	char *from;
+	const char *from;
 {
-	char *p, *e = to;
+	const char *p;
+	char *e = to;
 
 	for (p = from; *p; p++) {
 		if (*p == '%' || *p == ' ' || *p == '\t') {
