@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2002, 2003, 2004 Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2003, 2004
+ *	Tama Communications Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,15 +42,15 @@ const char *gozillarc = ".gozillarc";
 const char *dos_gozillarc = "_gozillarc";
 #endif
 
-static char *alias(char *);
+static const char *alias(const char *);
 int main(int, char **);
-void getdefinitionURL(char *, STRBUF *);
-void getURL(char *, STRBUF *);
-int isprotocol(char *);
-int issource(char *);
-int convertpath(char *, char *, char *, STRBUF *);
+void getdefinitionURL(const char *, STRBUF *);
+void getURL(const char *, STRBUF *);
+int isprotocol(const char *);
+int issource(const char *);
+int convertpath(const char *, const char *, const char *, STRBUF *);
 #if !defined(_WIN32) && !defined(__DJGPP__)
-void sendbrowser(char *, char *);
+void sendbrowser(const char *, const char *);
 #endif
 
 #ifndef isblank
@@ -66,14 +66,14 @@ int linenumber = 0;
 int debug;
 
 static void
-usage()
+usage(void)
 {
 	if (!qflag)
 		fputs(usage_const, stderr);
 	exit(2);
 }
 static void
-help()
+help(void)
 {
 	fputs(usage_const, stdout);
 	fputs(help_const, stdout);
@@ -92,13 +92,14 @@ help()
  * |f = file:/usr/share/xxx.html
  * |www	http://www.xxx.yyy/
  */
-static char *
+static const char *
 alias(alias_name)
-	char *alias_name;
+	const char *alias_name;
 {
 	FILE *ip;
 	STRBUF *sb = strbuf_open(0);
-	char *p, *alias = NULL;
+	char *p;
+	const char *alias = NULL;
 	int flag = STRBUF_NOCRLF;
 
 	if (!(p = get_home_directory()))
@@ -156,9 +157,8 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	char c, *p;
-	char *browser = NULL;
-	char *definition = NULL;
+	char c;
+	const char *p, *browser = NULL, *definition = NULL;
 	STRBUF *arg = strbuf_open(0);
 	STRBUF *URL = strbuf_open(0);
 
@@ -294,7 +294,7 @@ main(argc, argv)
  */
 void
 getdefinitionURL(arg, URL)
-	char *arg;
+	const char *arg;
 	STRBUF *URL;
 {
 	char cwd[MAXPATHLEN+1];
@@ -328,7 +328,7 @@ getdefinitionURL(arg, URL)
 		dbop = dbop_open(path, 0, 0, 0);
 	}
 	if (dbop) {
-		if ((p = dbop_get(dbop, arg)) != NULL) {
+		if ((p = (char *)dbop_get(dbop, arg)) != NULL) {
 			if (split(p, 2, &ptable) != 2)
 				die("illegal format.");
 			status = 0;
@@ -376,7 +376,7 @@ getdefinitionURL(arg, URL)
  */
 void
 getURL(arg, URL)
-	char *arg;
+	const char *arg;
 	STRBUF *URL;
 {
 	char cwd[MAXPATHLEN+1];
@@ -446,9 +446,9 @@ getURL(arg, URL)
  */
 int
 isprotocol(url)
-	char *url;
+	const char *url;
 {
-	char *p;
+	const char *p;
 
 	if (!strncmp(url, "file:", 5))
 		return 1;
@@ -472,7 +472,7 @@ isprotocol(url)
  */
 int
 issource(path)
-	char *path;
+	const char *path;
 {
 	STRBUF *sb = strbuf_open(0);
 	char *p;
@@ -509,15 +509,15 @@ issource(path)
  */
 int
 convertpath(dbpath, htmldir, path, sb)
-	char *dbpath;
-	char *htmldir;
-	char *path;
+	const char *dbpath;
+	const char *htmldir;
+	const char *path;
 	STRBUF *sb;
 {
 	static const char *suffix[] = {".html", ".htm"};
 	static const char *gz = ".gz";
 	int i, lim = sizeof(suffix)/sizeof(char *);
-	char *p;
+	const char *p;
 
 	strbuf_reset(sb);
 	strbuf_puts(sb, htmldir);
@@ -578,8 +578,8 @@ convertpath(dbpath, htmldir, path, sb)
  */
 void
 sendbrowser(browser, url)
-	char *browser;
-	char *url;
+	const char *browser;
+	const char *url;
 {
 	int pid;
 	char com[1024], *path;
