@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,7 @@
 #include "token.h"
 #include "c_res.h"
 
+static void C_family(const char *, int);
 static void process_attribute(int);
 static int function_definition(int, char *);
 static void condition_macro(int, int);
@@ -55,6 +56,9 @@ static int seems_datatype(const char *);
 #define RULES           1
 #define PROGRAMS        2
 
+#define TYPE_C		0
+#define TYPE_LEX	1
+#define TYPE_YACC	2
 /*
  * #ifdef stack.
  */
@@ -67,7 +71,6 @@ static struct {
 } stack[MAXPIFSTACK], *cur;
 static int piflevel;		/* condition macro level */
 static int level;		/* brace level */
-static int yaccflag;		/* 1: yacc, 0: c */
 
 /*
  * yacc: read yacc file and pickup tag entries.
@@ -76,8 +79,7 @@ void
 yacc(file)
 	const char *file;
 {
-	yaccflag = 1;
-	C(file);
+	C_family(file, TYPE_YACC);
 }
 /*
  * C: read C file and pickup tag entries.
@@ -85,6 +87,17 @@ yacc(file)
 void
 C(file)
 	const char *file;
+{
+	C_family(file, TYPE_C);
+}
+/*
+ *	i)	file	source file
+ *	i)	type	TYPE_C, TYPE_YACC, TYPE_LEX
+ */
+static void
+C_family(file, type)
+	const char *file;
+	int type;
 {
 	int c, cc;
 	int savelevel;
@@ -102,8 +115,8 @@ C(file)
 	 * programs
 	 *
 	 */
-	int yaccstatus = (yaccflag) ? DECLARATIONS : PROGRAMS;
-	int inC = (yaccflag) ? 0 : 1;		/* 1 while C source */
+	int yaccstatus = (type == TYPE_YACC) ? DECLARATIONS : PROGRAMS;
+	int inC = (type == TYPE_YACC) ? 0 : 1;	/* 1 while C source */
 
 	level = piflevel = 0;
 	savelevel = -1;
@@ -114,10 +127,10 @@ C(file)
 		die("'%s' cannot open.", file);
 	cmode = 1;			/* allow token like '#xxx' */
 	crflag = 1;			/* require '\n' as a token */
-	if (yaccflag)
+	if (type == TYPE_YACC)
 		ymode = 1;		/* allow token like '%xxx' */
 
-	while ((cc = nexttoken(interested, reserved_word)) != EOF) {
+	while ((cc = nexttoken(interested, c_reserved_word)) != EOF) {
 		switch (cc) {
 		case SYMBOL:		/* symbol	*/
 			if (inC && peekc(0) == '('/* ) */) {
@@ -261,14 +274,14 @@ C(file)
 		case SHARP_UNDEF:
 			startmacro = 1;
 			savelevel = level;
-			if ((c = nexttoken(interested, reserved_word)) != SYMBOL) {
+			if ((c = nexttoken(interested, c_reserved_word)) != SYMBOL) {
 				pushbacktoken();
 				break;
 			}
 			if (peekc(1) == '('/* ) */) {
 				if (target == DEF)
 					PUT(token, lineno, sp);
-				while ((c = nexttoken("()", reserved_word)) != EOF && c != '\n' && c != /* ( */ ')')
+				while ((c = nexttoken("()", c_reserved_word)) != EOF && c != '\n' && c != /* ( */ ')')
 					if (c == SYMBOL && target == SYM)
 						PUT(token, lineno, sp);
 				if (c == '\n')
@@ -292,7 +305,7 @@ C(file)
 		case SHARP_WARNING:
 		case SHARP_IDENT:
 		case SHARP_SCCS:
-			while ((c = nexttoken(interested, reserved_word)) != EOF && c != '\n')
+			while ((c = nexttoken(interested, c_reserved_word)) != EOF && c != '\n')
 				;
 			break;
 		case SHARP_IFDEF:
@@ -304,10 +317,10 @@ C(file)
 			condition_macro(cc, target);
 			break;
 		case SHARP_SHARP:		/* ## */
-			(void)nexttoken(interested, reserved_word);
+			(void)nexttoken(interested, c_reserved_word);
 			break;
 		case C_STRUCT:
-			c = nexttoken(interested, reserved_word);
+			c = nexttoken(interested, c_reserved_word);
 			if (c == '{' /* } */) {
 				pushbacktoken();
 				break;
@@ -342,7 +355,7 @@ C(file)
 
 				/* skip type qualifiers */
 				do {
-					c = nexttoken("{}(),;", reserved_word);
+					c = nexttoken("{}(),;", c_reserved_word);
 				} while (IS_TYPE_QUALIFIER(c) || c == '\n');
 
 				if (wflag && c == EOF) {
@@ -352,14 +365,14 @@ C(file)
 					char *interest_enum = "{},;";
 					int c_ = c;
 
-					c = nexttoken(interest_enum, reserved_word);
+					c = nexttoken(interest_enum, c_reserved_word);
 					/* read enum name if exist */
 					if (c == SYMBOL) {
 						if (target == SYM)
 							PUT(token, lineno, sp);
-						c = nexttoken(interest_enum, reserved_word);
+						c = nexttoken(interest_enum, c_reserved_word);
 					}
-					for (; c != EOF; c = nexttoken(interest_enum, reserved_word)) {
+					for (; c != EOF; c = nexttoken(interest_enum, c_reserved_word)) {
 						switch (c) {
 						case SHARP_IFDEF:
 						case SHARP_IFNDEF:
@@ -415,7 +428,7 @@ C(file)
 						PUT(token, lineno, sp);
 				}
 				savetok[0] = 0;
-				while ((c = nexttoken("(),;", reserved_word)) != EOF) {
+				while ((c = nexttoken("(),;", c_reserved_word)) != EOF) {
 					switch (c) {
 					case SHARP_IFDEF:
 					case SHARP_IFNDEF:
@@ -495,7 +508,7 @@ process_attribute(target)
 	 * Skip '...' in __attribute__((...))
 	 * but pick up symbols in it.
 	 */
-	while ((c = nexttoken("()", reserved_word)) != EOF) {
+	while ((c = nexttoken("()", c_reserved_word)) != EOF) {
 		if (c == '(')
 			brace++;
 		else if (c == ')')
@@ -530,7 +543,7 @@ function_definition(target, arg1)
 	int accept_arg1 = 0;
 
 	brace_level = isdefine = 0;
-	while ((c = nexttoken("()", reserved_word)) != EOF) {
+	while ((c = nexttoken("()", c_reserved_word)) != EOF) {
 		switch (c) {
 		case SHARP_IFDEF:
 		case SHARP_IFNDEF:
@@ -567,7 +580,7 @@ function_definition(target, arg1)
 	if (c == EOF)
 		return 0;
 	brace_level = 0;
-	while ((c = nexttoken(",;[](){}=", reserved_word)) != EOF) {
+	while ((c = nexttoken(",;[](){}=", c_reserved_word)) != EOF) {
 		switch (c) {
 		case SHARP_IFDEF:
 		case SHARP_IFNDEF:
@@ -637,7 +650,7 @@ condition_macro(cc, target)
 		cur->if0only = 0;
 		if (peekc(0) == '0')
 			cur->if0only = 1;
-		else if ((cc = nexttoken(NULL, reserved_word)) == SYMBOL && !strcmp(token, "notdef"))
+		else if ((cc = nexttoken(NULL, c_reserved_word)) == SYMBOL && !strcmp(token, "notdef"))
 			cur->if0only = 1;
 		else
 			pushbacktoken();
@@ -671,7 +684,7 @@ condition_macro(cc, target)
 			}
 		}
 	}
-	while ((cc = nexttoken(NULL, reserved_word)) != EOF && cc != '\n') {
+	while ((cc = nexttoken(NULL, c_reserved_word)) != EOF && cc != '\n') {
                 if (cc == SYMBOL && strcmp(token, "defined") != 0) {
 			if (target == REF) {
 				if (dflag && defined(token))
