@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -105,7 +105,7 @@ Cpp(file)
 	crflag = 1;			/* require '\n' as a token */
 	cppmode = 1;			/* treat '::' as a token */
 
-	while ((cc = nexttoken(interested, reserved_word)) != EOF) {
+	while ((cc = nexttoken(interested, cpp_reserved_word)) != EOF) {
 		if (cc == '~' && level == stack[classlevel].level)
 			continue;
 		switch (cc) {
@@ -158,8 +158,8 @@ Cpp(file)
 			}
 			break;
 		case CPP_USING:
-			if ((c = nexttoken(interested, reserved_word)) == CPP_NAMESPACE) {
-				if ((c = nexttoken(interested, reserved_word)) == SYMBOL) {
+			if ((c = nexttoken(interested, cpp_reserved_word)) == CPP_NAMESPACE) {
+				if ((c = nexttoken(interested, cpp_reserved_word)) == SYMBOL) {
 					if (target == REF)
 						PUT(token, lineno, sp);
 				} else {
@@ -172,11 +172,11 @@ Cpp(file)
 			/* FALLTHROUGH */
 		case CPP_NAMESPACE:
 			if (peekc(0) != '{') /* } */ {
-				if ((c = nexttoken(interested, reserved_word)) == SYMBOL)
+				if ((c = nexttoken(interested, cpp_reserved_word)) == SYMBOL)
 					if (target == DEF)
 						PUT(token, lineno, sp);
 			}
-			while ((c = nexttoken(interested, reserved_word)) == '\n')
+			while ((c = nexttoken(interested, cpp_reserved_word)) == '\n')
 				;
 			/*
 			 * Namespace block doesn't have any influence on level.
@@ -190,7 +190,7 @@ Cpp(file)
 			break;
 		case CPP_CLASS:
 			DBG_PRINT(level, "class");
-			if ((c = nexttoken(interested, reserved_word)) == SYMBOL) {
+			if ((c = nexttoken(interested, cpp_reserved_word)) == SYMBOL) {
 				strlimcpy(classname, token, sizeof(classname));
 				if (target == DEF)
 					PUT(token, lineno, sp);
@@ -264,14 +264,14 @@ Cpp(file)
 		case SHARP_UNDEF:
 			startmacro = 1;
 			savelevel = level;
-			if ((c = nexttoken(interested, reserved_word)) != SYMBOL) {
+			if ((c = nexttoken(interested, cpp_reserved_word)) != SYMBOL) {
 				pushbacktoken();
 				break;
 			}
 			if (peekc(1) == '('/* ) */) {
 				if (target == DEF)
 					PUT(token, lineno, sp);
-				while ((c = nexttoken("()", reserved_word)) != EOF && c != '\n' && c != /* ( */ ')')
+				while ((c = nexttoken("()", cpp_reserved_word)) != EOF && c != '\n' && c != /* ( */ ')')
 					if (c == SYMBOL && target == SYM)
 						PUT(token, lineno, sp);
 				if (c == '\n')
@@ -295,7 +295,7 @@ Cpp(file)
 		case SHARP_WARNING:
 		case SHARP_IDENT:
 		case SHARP_SCCS:
-			while ((c = nexttoken(interested, reserved_word)) != EOF && c != '\n')
+			while ((c = nexttoken(interested, cpp_reserved_word)) != EOF && c != '\n')
 				;
 			break;
 		case SHARP_IFDEF:
@@ -307,15 +307,15 @@ Cpp(file)
 			condition_macro(cc, target);
 			break;
 		case SHARP_SHARP:		/* ## */
-			(void)nexttoken(interested, reserved_word);
+			(void)nexttoken(interested, cpp_reserved_word);
 			break;
 		case CPP_NEW:
-			if ((c = nexttoken(interested, reserved_word)) == SYMBOL)
+			if ((c = nexttoken(interested, cpp_reserved_word)) == SYMBOL)
 				if (target == REF && defined(token))
 					PUT(token, lineno, sp);
 			break;
 		case CPP_STRUCT:
-			c = nexttoken(interested, reserved_word);
+			c = nexttoken(interested, cpp_reserved_word);
 			if (c == '{' /* } */) {
 				pushbacktoken();
 				break;
@@ -328,7 +328,7 @@ Cpp(file)
 			{
 				int level = 0;
 
-				while ((c = nexttoken("<>", reserved_word)) != EOF) {
+				while ((c = nexttoken("<>", cpp_reserved_word)) != EOF) {
 					if (c == '<')
 						++level;
 					else if (c == '>') {
@@ -344,7 +344,7 @@ Cpp(file)
 			}
 			break;
 		case CPP_OPERATOR:
-			while ((c = nexttoken(";{", /* } */ reserved_word)) != EOF) {
+			while ((c = nexttoken(";{", /* } */ cpp_reserved_word)) != EOF) {
 				if (c == '{') /* } */ {
 					pushbacktoken();
 					break;
@@ -389,7 +389,7 @@ Cpp(file)
 
 				/* skip CV qualifiers */
 				do {
-					c = nexttoken("{}(),;", reserved_word);
+					c = nexttoken("{}(),;", cpp_reserved_word);
 				} while (IS_CV_QUALIFIER(c) || c == '\n');
 
 				if (wflag && c == EOF) {
@@ -399,14 +399,14 @@ Cpp(file)
 					char *interest_enum = "{},;";
 					int c_ = c;
 
-					c = nexttoken(interest_enum, reserved_word);
+					c = nexttoken(interest_enum, cpp_reserved_word);
 					/* read enum name if exist */
 					if (c == SYMBOL) {
 						if (target == SYM)
 							PUT(token, lineno, sp);
-						c = nexttoken(interest_enum, reserved_word);
+						c = nexttoken(interest_enum, cpp_reserved_word);
 					}
-					for (; c != EOF; c = nexttoken(interest_enum, reserved_word)) {
+					for (; c != EOF; c = nexttoken(interest_enum, cpp_reserved_word)) {
 						switch (c) {
 						case SHARP_IFDEF:
 						case SHARP_IFNDEF:
@@ -462,7 +462,7 @@ Cpp(file)
 						PUT(token, lineno, sp);
 				}
 				savetok[0] = 0;
-				while ((c = nexttoken("(),;", reserved_word)) != EOF) {
+				while ((c = nexttoken("(),;", cpp_reserved_word)) != EOF) {
 					switch (c) {
 					case SHARP_IFDEF:
 					case SHARP_IFNDEF:
@@ -542,7 +542,7 @@ process_attribute(target)
 	 * Skip '...' in __attribute__((...))
 	 * but pick up symbols in it.
 	 */
-	while ((c = nexttoken("()", reserved_word)) != EOF) {
+	while ((c = nexttoken("()", cpp_reserved_word)) != EOF) {
 		if (c == '(')
 			brace++;
 		else if (c == ')')
@@ -573,7 +573,7 @@ function_definition(target)
 	int brace_level;
 
 	brace_level = 0;
-	while ((c = nexttoken("()", reserved_word)) != EOF) {
+	while ((c = nexttoken("()", cpp_reserved_word)) != EOF) {
 		switch (c) {
 		case SHARP_IFDEF:
 		case SHARP_IFNDEF:
@@ -610,7 +610,7 @@ function_definition(target)
 		return 0;
 	}
 	brace_level = 0;
-	while ((c = nexttoken(",;[](){}=", reserved_word)) != EOF) {
+	while ((c = nexttoken(",;[](){}=", cpp_reserved_word)) != EOF) {
 		switch (c) {
 		case SHARP_IFDEF:
 		case SHARP_IFNDEF:
@@ -675,7 +675,7 @@ condition_macro(cc, target)
 		cur->if0only = 0;
 		if (peekc(0) == '0')
 			cur->if0only = 1;
-		else if ((cc = nexttoken(NULL, reserved_word)) == SYMBOL && !strcmp(token, "notdef"))
+		else if ((cc = nexttoken(NULL, cpp_reserved_word)) == SYMBOL && !strcmp(token, "notdef"))
 			cur->if0only = 1;
 		else
 			pushbacktoken();
@@ -709,7 +709,7 @@ condition_macro(cc, target)
 			}
 		}
 	}
-	while ((cc = nexttoken(NULL, reserved_word)) != EOF && cc != '\n') {
+	while ((cc = nexttoken(NULL, cpp_reserved_word)) != EOF && cc != '\n') {
                 if (cc == SYMBOL && strcmp(token, "defined") != 0) {
 			if (target == REF) {
 				if (dflag && defined(token))
