@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,11 +16,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include <ctype.h>
 #include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -44,93 +45,96 @@
  */
 int
 makedefineindex(file, total, defines)
-	char *file;
+	const char *file;
 	int total;
 	STRBUF *defines;
 {
 	int count = 0;
 	int alpha_count = 0;
-	char indexlink[1024];
-	char *index_string = "Index Page";
-	char *target = (Fflag) ? "mains" : "_top";
 	FILE *MAP = NULL;
-	FILE *DEFINES, *old, *STDOUT, *TAGS, *ALPHA = NULL;
+	FILE *DEFINES, *STDOUT, *TAGS, *ALPHA = NULL;
 	STRBUF *sb = strbuf_open(0);
 	STRBUF *url = strbuf_open(0);
-	char *_;
-	char command[1024], buf[1024], alpha[32], alpha_f[32];
+	/* Index link */
+	const char *target = (Fflag) ? "mains" : "_top";
+	const char *indexlink;
+	const char *index_string = "Index Page";
+	char command[1024], buf[1024], alpha[32], alpha_f[32], *_;
 
-	if (Fflag)
-		snprintf(indexlink, sizeof(indexlink), "../defines.%s", normal_suffix);
+	if (!aflag && !Fflag)
+		indexlink = "mains";
+	else if (Fflag)
+		indexlink = "../defines";
 	else
-		snprintf(indexlink, sizeof(indexlink), "../mains.%s", normal_suffix);
+		indexlink = "../mains";
+
 	if (map_file) {
 		if (!(MAP = fopen(makepath(distpath, "MAP", NULL), "w")))
 			die("cannot open '%s'.", makepath(distpath, "MAP", NULL));
 	}
 	if (!(DEFINES = fopen(makepath(distpath, file, NULL), "w")))
 		die("cannot make function index '%s'.", file);
-	fprintf(DEFINES, "%s\n", html_begin);
-	fprintf(DEFINES, "%s", set_header(title_define_index));
-	fprintf(DEFINES, "%s\n", body_begin);
+	fputs_nl(gen_page_begin(title_define_index, TOPDIR), DEFINES);
+	fputs_nl(body_begin, DEFINES);
+	fputs(header_begin, DEFINES);
 	if (Fflag)
-		fprintf(DEFINES, "<A HREF=defines.%s><H2>%s</H2></A>\n", normal_suffix, title_define_index);
-	else
-		fprintf(DEFINES, "<H2>%s</H2>\n", title_define_index);
+		fputs(gen_href_begin(NULL, "defines", normal_suffix, NULL), DEFINES);
+	fputs(title_define_index, DEFINES);
+	if (Fflag)
+		fputs(gen_href_end(), DEFINES);
+	fputs_nl(header_end, DEFINES);
 	if (!aflag && !Fflag) {
-		snprintf(indexlink, sizeof(indexlink), "mains.%s", normal_suffix);
-		fprintf(DEFINES, "<A HREF=%s TITLE='%s'>", indexlink, index_string);
+		fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), DEFINES);
 		if (icon_list)
-			fprintf(DEFINES, "<IMG SRC=icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+			fputs(gen_image(CURRENT, back_icon, ".."), DEFINES);
 		else
-			fprintf(DEFINES, "[..]");
-		fprintf(DEFINES, "</A>\n");
+			fputs("[..]", DEFINES);
+		fputs_nl(gen_href_end(), DEFINES);
 	}
 	if (!aflag) {
 		if (!no_order_list)
-			fprintf(DEFINES, "%s\n", list_begin);
-		/*
-		else
-			fprintf(DEFINES, "%s\n", br);
-		*/
+			fputs_nl(list_begin, DEFINES);
 	}
 	/*
 	 * map DEFINES to STDOUT.
 	 */
-	old = STDOUT;
 	STDOUT = DEFINES;
 	snprintf(command, sizeof(command), "global -c");
 	if ((TAGS = popen(command, "r")) == NULL)
 		die("cannot fork.");
 	alpha[0] = '\0';
 	while ((_ = strbuf_fgets(sb, TAGS, STRBUF_NOCRLF)) != NULL) {
-		char *line, *tag;
+		const char *tag, *line;
 		char guide[1024], url_for_map[1024];
 
 		count++;
 		tag = _;
 		message(" [%d/%d] adding %s", count, total, tag);
 		if (aflag && (alpha[0] == '\0' || strncmp(tag, alpha, strlen(alpha)))) {
-			char *msg = (alpha_count == 1) ? "definition is containded." : "definitions are containded.";
+			const char *msg = (alpha_count == 1) ? "definition is contained." : "definitions are contained.";
+			int c;
 
 			if (alpha[0]) {
-				strbuf_sprintf(defines, "<A HREF=defines/%s.%s TITLE='%d %s'>[%s]</A>\n",
-					alpha_f, HTML, alpha_count, msg, alpha);
+				char tmp[128];
+				snprintf(tmp, sizeof(tmp), "%d %s", alpha_count, msg);
+				strbuf_puts(defines, gen_href_begin_with_title("defines", alpha_f, HTML, NULL, tmp));
+				strbuf_sprintf(defines, "[%s]", alpha);
+				strbuf_puts_nl(defines, gen_href_end());
 				alpha_count = 0;
 				if (!no_order_list)
-					fprintf(ALPHA, "%s\n", list_end);
+					fputs_nl(list_end, ALPHA);
 				else
-					fprintf(ALPHA, "%s\n", br);
-				fprintf(ALPHA, "<A HREF=%s TITLE='%s'>", indexlink, index_string);
+					fputs_nl(br, ALPHA);
+				fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), ALPHA);
 				if (icon_list)
-					fprintf(ALPHA, "<IMG SRC=../icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+					fputs(gen_image(PARENT, back_icon, ".."), ALPHA);
 				else
-					fprintf(ALPHA, "[..]");
-				fprintf(ALPHA, "</A>\n");
-				fprintf(ALPHA, "%s\n", body_end);
-				fprintf(ALPHA, "%s\n", html_end);
+					fputs("[..]", ALPHA);
+				fputs_nl(gen_href_end(), ALPHA);
+				fputs_nl(body_end, ALPHA);
+				fputs_nl(gen_page_end(), ALPHA);
 				if (cflag) {
-					if (pclose(ALPHA) < 0)
+					if (pclose(ALPHA) != 0)
 						die("terminated abnormally.");
 				} else
 					fclose(ALPHA);
@@ -141,8 +145,8 @@ makedefineindex(file, total, defines)
 			 * alpha is used for display.
 			 * alpha_f is used for part of path.
 			 */
-			if (*(unsigned char *)tag > 127) {
-				int i1 = *tag & 0xff;
+			c = (unsigned char)*tag;
+			if (c > 127) {
 				int i2 = *(tag + 1) & 0xff;
 				/*
 				 * for multi-byte(EUC) code.
@@ -150,15 +154,14 @@ makedefineindex(file, total, defines)
 				alpha[0] = *tag;
 				alpha[1] = *(tag + 1);
 				alpha[2] = '\0';
-				snprintf(alpha_f, sizeof(alpha_f), "%03d%03d", i1, i2);
-			} else {
+				snprintf(alpha_f, sizeof(alpha_f), "%03d%03d", c, i2);
+			} else if (isalpha(c) || c == '_') {
 				alpha[0] = *tag;
 				alpha[1] = '\0';
 				/*
 				 * for CD9660 or FAT file system
-				 * 97 == 'a', 122 == 'z'
 				 */
-				if (*tag >= 'a' && *tag <= 'z') {
+				if (islower(c)) {
 					alpha_f[0] = 'l';
 					alpha_f[1] = *tag;
 					alpha_f[2] = '\0';
@@ -166,6 +169,10 @@ makedefineindex(file, total, defines)
 					alpha_f[0] = *tag;
 					alpha_f[1] = '\0';
 				}
+			} else {
+				alpha[0] = *tag;
+				alpha[1] = '\0';
+				snprintf(alpha_f, sizeof(alpha_f), "%03d", c);
 			}
 			if (cflag) {
 				snprintf(buf, sizeof(buf), "gzip -c >%s/defines/%s.%s", distpath, alpha_f, HTML);
@@ -176,19 +183,18 @@ makedefineindex(file, total, defines)
 			}
 			if (!ALPHA)
 				die("cannot make alphabetical function index.");
-			fprintf(ALPHA, "%s\n", html_begin);
 			snprintf(buf, sizeof(buf), "[%s]", alpha);
-			fprintf(ALPHA, "%s", set_header(buf));
-			fprintf(ALPHA, "%s\n", body_begin);
-			fprintf(ALPHA, "<H2>[%s]</H2>\n", alpha);
-			fprintf(ALPHA, "<A HREF=%s TITLE='%s'>", indexlink, index_string);
+			fputs_nl(gen_page_begin(buf, SUBDIR), ALPHA);
+			fputs_nl(body_begin, ALPHA);
+			fprintf(ALPHA, "%s[%s]%s\n", header_begin, alpha, header_end);
+			fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), ALPHA);
 			if (icon_list)
-				fprintf(ALPHA, "<IMG SRC=../icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+				fputs(gen_image(PARENT, back_icon, ".."), ALPHA);
 			else
-				fprintf(ALPHA, "[..]");
-			fprintf(ALPHA, "</A>\n");
+				fputs("[..]", ALPHA);
+			fputs_nl(gen_href_end(), ALPHA);
 			if (!no_order_list)
-				fprintf(ALPHA, "%s\n", list_begin);
+				fputs_nl(list_begin, ALPHA);
 			else
 				fprintf(ALPHA, "%s%s\n", br, br);
 			STDOUT = ALPHA;
@@ -204,9 +210,9 @@ makedefineindex(file, total, defines)
 			die("internal error in makedefineindex()."); 
 		if (*line == ' ') {
 			SPLIT ptable;
-			char *fid, *enumber;
+			const char *fid, *enumber;
 
-			if (split(line + 1, 2, &ptable) < 2) {
+			if (split((char *)line + 1, 2, &ptable) < 2) {
 				recover(&ptable);
 				die("too small number of parts in makedefineindex().\n'%s'", line);
 			}
@@ -221,7 +227,7 @@ makedefineindex(file, total, defines)
 				if (*action != '/' && aflag)
 					strbuf_puts(url, "../");
 				strbuf_puts(url, action);
-				strbuf_sprintf(url, "?pattern=%s&type=definitions", tag);
+				strbuf_sprintf(url, "?pattern=%s%stype=definitions", tag, quote_amp);
 			} else {
 				if (aflag)
 					strbuf_puts(url, "../");
@@ -231,9 +237,9 @@ makedefineindex(file, total, defines)
 			recover(&ptable);
 		} else {
 			SPLIT ptable;
-			char *lno, *filename, *path;
+			const char *lno, *fid, *path;
 
-			if (split(line, 3, &ptable) < 3) {
+			if (split((char *)line, 3, &ptable) < 3) {
 				recover(&ptable);
 				die("too small number of parts in makedefineindex().\n'%s'", line);
 			}
@@ -241,59 +247,67 @@ makedefineindex(file, total, defines)
 			path = ptable.part[1].start;
 			path += 2;		/* remove './' */
 
-			filename = path2url(path);
-			snprintf(url_for_map, sizeof(url_for_map), "%s/%s#%s",
-				SRCS, filename, lno);
+			fid = path2fid(path);
+			snprintf(url_for_map, sizeof(url_for_map), "%s/%s.%s#L%s",
+				SRCS, fid, HTML, lno);
 			if (aflag)
 				strbuf_puts(url, "../");
-			strbuf_sprintf(url, "%s/%s#%s", SRCS, filename, lno);
+			strbuf_sprintf(url, "%s/%s.%s#L%s", SRCS, fid, HTML, lno);
 			snprintf(guide, sizeof(guide), "Defined at %s in %s.", lno, path);
 			recover(&ptable);
 		}
 		if (!no_order_list)
-			fprintf(STDOUT, list_item);
-		fprintf(STDOUT, "<A HREF=%s TARGET=%s TITLE='%s'>%s</A>\n", strbuf_value(url), target, guide, tag);
-		if (no_order_list)
-			fprintf(STDOUT, br);
+			fputs(item_begin, STDOUT);
+		fputs(gen_href_begin_with_title_target(NULL, strbuf_value(url), NULL, NULL, guide, target), STDOUT);
+		fputs(tag, STDOUT);
+		fputs(gen_href_end(), STDOUT);
+		if (!no_order_list)
+			fputs(item_end, STDOUT);
+		else
+			fputs(br, STDOUT);
+		fputc('\n', STDOUT);
 		if (map_file)
 			fprintf(MAP, "%s\t%s\n", tag, url_for_map);
 	}
-	if (pclose(TAGS) < 0)
+	if (pclose(TAGS) != 0)
 		die("'%s' failed.", command);
-	STDOUT = old;
 	if (aflag && alpha[0]) {
-		char *msg = (alpha_count == 1) ? "definition is containded." : "definitions are containded.";
+		char tmp[128];
+		const char *msg = (alpha_count == 1) ? "definition is contained." : "definitions are contained.";
 
-		strbuf_sprintf(defines, "<A HREF=defines/%s.%s TITLE='%d %s'>[%s]</A>\n", alpha_f, HTML, alpha_count, msg, alpha);
+		snprintf(tmp, sizeof(tmp), "%d %s", alpha_count, msg);
+		strbuf_puts(defines, gen_href_begin_with_title("defines", alpha_f, HTML, NULL, tmp));
+		strbuf_sprintf(defines, "[%s]", alpha);
+		strbuf_puts_nl(defines, gen_href_end());
 		if (!no_order_list)
-			fprintf(ALPHA, "%s\n", list_end);
+			fputs_nl(list_end, ALPHA);
 		else
-			fprintf(ALPHA, "%s\n", br);
-		fprintf(ALPHA, "<A HREF=%s TITLE='%s'>", indexlink, index_string);
+			fputs_nl(br, ALPHA);
+		fputs(gen_href_begin_with_title(NULL, indexlink, normal_suffix, NULL, index_string), ALPHA);
 		if (icon_list)
-			fprintf(ALPHA, "<IMG SRC=../icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+			fputs(gen_image(PARENT, back_icon, ".."), ALPHA);
 		else
-			fprintf(ALPHA, "[..]");
-		fprintf(ALPHA, "</A>\n");
-		fprintf(ALPHA, "%s\n", body_end);
-		fprintf(ALPHA, "%s\n", html_end);
+			fputs("[..]", ALPHA);
+		fputs_nl(gen_href_end(), ALPHA);
+		fputs_nl(body_end, ALPHA);
+		fputs_nl(gen_page_end(), ALPHA);
 		fclose(ALPHA);
 		file_count++;
 
-		fprintf(DEFINES, strbuf_value(defines));
+		fputs(strbuf_value(defines), DEFINES);
 	}
 	if (!no_order_list && !aflag)
-		fprintf(DEFINES, "%s\n", list_end);
+		fputs_nl(list_end, DEFINES);
 	if (!aflag && !Fflag) {
-		fprintf(DEFINES, "<A HREF=mains.%s TITLE='Index Page'>", normal_suffix);
+		fputs(gen_href_begin_with_title(NULL, "mains", normal_suffix, NULL, index_string), DEFINES);
 		if (icon_list)
-			fprintf(DEFINES, "<IMG SRC=icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+			fputs(gen_image(CURRENT, back_icon, ".."), DEFINES);
 		else
-			fprintf(DEFINES, "[..]");
-		fprintf(DEFINES, "</A>\n");
+			fputs("[..]", DEFINES);
+		fputs_nl(gen_href_end(), DEFINES);
 	}
-	fprintf(DEFINES, "%s\n", body_end);
-	fprintf(DEFINES, "%s\n", html_end);
+	fputs_nl(body_end, DEFINES);
+	fputs_nl(gen_page_end(), DEFINES);
 	fclose(DEFINES);
 	file_count++;
 	if (map_file)
