@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -36,11 +36,11 @@
 /*
  * Data for each tag file.
  *
- *				GTAGS         GRTAGS       GSYMS
+ *				      GTAGS         GRTAGS       GSYMS
  */
-static char *dirs[]    = {NULL, DEFS,         REFS,        SYMS};
-static char *kinds[]   = {NULL, "definition", "reference", "symbol"};
-static char *options[] = {NULL, "",           "r",         "s"};
+static const char *dirs[]    = {NULL, DEFS,         REFS,        SYMS};
+static const char *kinds[]   = {NULL, "definition", "reference", "symbol"};
+static const char *options[] = {NULL, "",           "r",         "s"};
 
 static char command[MAXFILLEN];
 
@@ -95,7 +95,7 @@ close_dup_file(op)
  * is generated.
  */
 int
-makedupindex()
+makedupindex(void)
 {
 	STRBUF *sb = strbuf_open(0);
 	int definition_count = 0;
@@ -107,15 +107,12 @@ makedupindex()
 
 	snprintf(srcdir, sizeof(srcdir), "../%s", SRCS);
 	for (db = GTAGS; db < GTAGLIM; db++) {
-		char *kind = kinds[db];
-		char *option = options[db];
-		char tag[IDENTLEN], prev[IDENTLEN];
-		char first_line[MAXBUFLEN];
+		const char *kind = kinds[db];
+		const char *option = options[db];
 		int writing = 0;
 		int count = 0;
 		int entry_count = 0;
-		char command[MAXFILLEN];
-		char *_;
+		char *ctags_x, tag[IDENTLEN], prev[IDENTLEN], first_line[MAXBUFLEN], command[MAXFILLEN];
 
 		if (!symbol && db == GSYMS)
 			continue;
@@ -124,14 +121,14 @@ makedupindex()
 		snprintf(command, sizeof(command), "global -xn%s%s \".*\" | gtags --sort", dynamic ? "n" : "", option);
 		if ((ip = popen(command, "r")) == NULL)
 			die("cannot execute command '%s'.", command);
-		while ((_ = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
+		while ((ctags_x = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
 			SPLIT ptable;
 
-			if (split(_, 2, &ptable) < 2) {
+			if (split(ctags_x, 2, &ptable) < 2) {
 				recover(&ptable);
-				die("too small number of parts.(1)\n'%s'", _);
+				die("too small number of parts.(1)\n'%s'", ctags_x);
 			}
-			strlimcpy(tag, ptable.part[0].start, sizeof(tag));
+			strlimcpy(tag, ptable.part[PART_TAG].start, sizeof(tag));
 			recover(&ptable);
 
 			if (strcmp(prev, tag)) {
@@ -140,9 +137,9 @@ makedupindex()
 					fprintf(stderr, " [%d] adding %s %s\n", count, kind, tag);
 				if (writing) {
 					if (!dynamic) {
-						fprintf(op, "%s\n", gen_list_end());
-						fprintf(op, "%s\n", body_end);
-						fprintf(op, "%s\n", html_end);
+						fputs_nl(gen_list_end(), op);
+						fputs_nl(body_end, op);
+						fputs_nl(gen_page_end(), op);
 						close_dup_file(op);
 						file_count++;
 					}
@@ -155,23 +152,19 @@ makedupindex()
 				}				
 				/* single entry */
 				if (first_line[0]) {
-					char *lno, *filename;
-
 					if (split(first_line, 3, &ptable) < 3) {
 						recover(&ptable);
-						die("too small number of parts.(2)\n'%s'", _);
+						die("too small number of parts.(2)\n'%s'", ctags_x);
 					}
-					lno = ptable.part[1].start;
-					filename = ptable.part[2].start;
-					snprintf(buf, sizeof(buf), "%s %s", lno, filename);
+					snprintf(buf, sizeof(buf), "%s %s", ptable.part[PART_LNO].start, ptable.part[PART_PATH].start);
 					cache_put(db, prev, buf);
 					recover(&ptable);
 				}
 				/*
 				 * Chop the tail of the line. It is not important.
-				 * strlimcpy(first_line, _, sizeof(first_line));
+				 * strlimcpy(first_line, ctags_x, sizeof(first_line));
 				 */
-				strncpy(first_line, _, sizeof(first_line));
+				strncpy(first_line, ctags_x, sizeof(first_line));
 				first_line[sizeof(first_line) - 1] = '\0';
 				strlimcpy(prev, tag, sizeof(prev));
 				entry_count = 0;
@@ -180,18 +173,17 @@ makedupindex()
 				if (first_line[0]) {
 					if (!dynamic) {
 						op = open_dup_file(db, count);
-						fprintf(op, "%s\n", html_begin);
-						fprintf(op, "%s", set_header(tag));
-						fprintf(op, "%s\n", body_begin);
-						fprintf(op, "%s\n", gen_list_begin());
-						fprintf(op, "%s\n", gen_list_body(srcdir, first_line));
+						fputs_nl(gen_page_begin(tag, SUBDIR), op);
+						fputs_nl(body_begin, op);
+						fputs_nl(gen_list_begin(), op);
+						fputs_nl(gen_list_body(srcdir, first_line), op);
 					}
 					writing = 1;
 					entry_count++;
 					first_line[0] = 0;
 				}
 				if (!dynamic) {
-					fprintf(op, "%s\n", gen_list_body(srcdir, _));
+					fputs_nl(gen_list_body(srcdir, ctags_x), op);
 				}
 				entry_count++;
 			}
@@ -202,9 +194,9 @@ makedupindex()
 			die("'%s' failed.", command);
 		if (writing) {
 			if (!dynamic) {
-				fprintf(op, "%s\n", gen_list_end());
-				fprintf(op, "%s\n", body_end);
-				fprintf(op, "%s\n", html_end);
+				fputs_nl(gen_list_end(), op);
+				fputs_nl(body_end, op);
+				fputs_nl(gen_page_end(), op);
 				close_dup_file(op);
 				file_count++;
 			}
@@ -216,15 +208,12 @@ makedupindex()
 		}
 		if (first_line[0]) {
 			SPLIT ptable;
-			char *lno, *filename;
 
 			if (split(first_line, 3, &ptable) < 3) {
 				recover(&ptable);
-				die("too small number of parts.(3)\n'%s'", _);
+				die("too small number of parts.(3)\n'%s'", ctags_x);
 			}
-			lno = ptable.part[1].start;
-			filename = ptable.part[2].start;
-			snprintf(buf, sizeof(buf), "%s %s", lno, filename);
+			snprintf(buf, sizeof(buf), "%s %s", ptable.part[PART_LNO].start, ptable.part[PART_PATH].start);
 			cache_put(db, prev, buf);
 			recover(&ptable);
 		}
