@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -38,6 +38,7 @@
 #include "path2url.h"
 #include "htags.h"
 #include "common.h"
+#include "test.h"
 
 /*----------------------------------------------------------------------*/
 /* File queue								*/
@@ -75,7 +76,7 @@ static SLIST_HEAD(, file) file_q;
  */
 static FILE *
 open_file_queue(path)
-	char *path;
+	const char *path;
 {
 	struct file *file = (struct file *)malloc(sizeof(struct file));
 
@@ -110,7 +111,7 @@ open_file_queue(path)
  */
 static FILE *
 select_file_queue(path)
-	char *path;
+	const char *path;
 {
 	struct file *file;
 
@@ -127,7 +128,7 @@ select_file_queue(path)
  */
 static void
 close_file_queue(path)
-	char *path;
+	const char *path;
 {
 	struct file *file;
 
@@ -204,7 +205,7 @@ struct dirstack {
 #define bottom_stack(sp) ((sp)->start)
 
 void
-settrace()
+settrace(void)
 {
 	trace = 1;
 }
@@ -216,21 +217,21 @@ settrace()
 void
 static dump_stack(sp, label)
 	struct dirstack *sp;
-	char *label;
+	const char *label;
 {
 	char *start = sp->start;
 	char *last = sp->last - 1;
-	char *p;
+	const char *p;
 
 	fprintf(stderr, "%s(%s): ", label, sp->name);
 	for (p = sp->buf; p < last; p++) {
 		if (p == start)
-			fprintf(stderr, "[");
+			fputs("[", stderr);
 		fputc((*p == 0) ? ' ' : *p, stderr);
 	}
 	if (start == sp->last)
-		fprintf(stderr, "[");
-	fprintf(stderr, "]\n");
+		fputs("[", stderr);
+	fputs_nl("]", stderr);
 }
 /*
  * make_stack: make new stack.
@@ -239,7 +240,7 @@ static dump_stack(sp, label)
  */
 static struct dirstack *
 make_stack(name)
-	char *name;
+	const char *name;
 {
 	struct dirstack *sp = (struct dirstack *)malloc(sizeof(struct dirstack));
 	if (!sp)
@@ -270,7 +271,7 @@ make_stack(name)
 static void
 set_stack(sp, path)
 	struct dirstack *sp;
-	char *path;
+	const char *path;
 {
 	int length = strlen(path) + 1;
 	char *p;
@@ -305,7 +306,7 @@ set_stack(sp, path)
 static void
 push_stack(sp, s)
 	struct dirstack *sp;
-	char *s;
+	const char *s;
 {
 	int length = strlen(s) + 1;
 
@@ -330,7 +331,7 @@ push_stack(sp, s)
  *	i)	sp	stack descriptor
  *	r)		string
  */
-static char *
+static const char *
 top_stack(sp)
 	struct dirstack *sp;
 {
@@ -359,10 +360,10 @@ top_stack(sp)
  *	i)	cur	current value
  *	r)		string
  */
-static char *
+static const char *
 next_stack(sp, cur)
 	struct dirstack *sp;
-	char *cur;
+	const char *cur;
 {
 	char *last = sp->last;
 
@@ -381,11 +382,11 @@ next_stack(sp, cur)
  *	i)	sp	stack descriptor
  *	r)		string
  */
-static char *
+static const char *
 pop_stack(sp)
 	struct dirstack *sp;
 {
-	char *last = top_stack(sp);
+	char *last = (char *)top_stack(sp);
 	int length = strlen(last) + 1;
 
 	if (!last)
@@ -403,7 +404,7 @@ pop_stack(sp)
  *	i)	sp	stack descriptor
  *	r)		string
  */
-static char *
+static const char *
 shift_stack(sp)
 	struct dirstack *sp;
 {
@@ -453,7 +454,7 @@ copy_stack(to, from)
  *	i)	sp	stack descriptor
  *	r)		path name
  */
-static char *
+static const char *
 join_stack(sp)
 	struct dirstack *sp;
 {
@@ -482,6 +483,31 @@ delete_stack(sp)
 /*----------------------------------------------------------------------*/
 /* Main procedure							*/
 /*----------------------------------------------------------------------*/
+/*
+ * Encode URL.
+ *
+ *	i)	url	URL
+ *	r)		encoded URL
+ */
+static const char *
+encode(url)
+        const char *url;
+{
+	STATIC_STRBUF(sb);
+        const char *p;
+
+	strbuf_clear(sb);
+        for (p = url; *p; p++) {
+		int c = (unsigned char)*p;
+
+                if (isalnum(c))
+			strbuf_putc(sb, c);
+		else
+			strbuf_sprintf(sb, "%%%02x", c);
+        }
+
+	return strbuf_value(sb);
+}
 /*
  * extract_lastname: extract the last name of include line.
  *
@@ -489,13 +515,14 @@ delete_stack(sp)
  *	i)	is_php	1: is PHP source
  *	r)		last name
  */
-static char *
+static const char *
 extract_lastname(image, is_php)
-	char *image;
+	const char *image;
 	int is_php;
 {
 	static char buf[MAXBUFLEN];
-	char *p, *q;
+	const char *p;
+	char *q;
 	int sep;
 
 	/*
@@ -561,12 +588,12 @@ extract_lastname(image, is_php)
 		*q++ = *p++;
 	*q = '\0';
 	if (*p == sep) {
-		p = locatestring(buf, "/", MATCH_LAST);
-		if (p)
-			p++;
+		q = locatestring(buf, "/", MATCH_LAST);
+		if (q)
+			q++;
 		else
-			p = buf;
-		return p;
+			q = buf;
+		return q;
 	}
 	return NULL;
 }
@@ -578,20 +605,19 @@ extract_lastname(image, is_php)
  */
 int
 makefileindex(file, files)
-	char *file;
+	const char *file;
 	STRBUF *files;
 {
 	FILE *FIND, *FILEMAP, *FILES, *STDOUT, *op = NULL;
 	char *_;
 	int count = 0;
-	char indexlink[80];
-	char edit[MAXURLLEN];
+	const char *indexlink = (Fflag) ? "../files" : "../mains";
 	STRBUF *sb = strbuf_open(0);
 	STRBUF *input = strbuf_open(0);
-	char *target = (Fflag) ? "mains" : "_top";
+	const char *target = (Fflag) ? "mains" : "_top";
 	struct dirstack *dirstack = make_stack("dirstack");
 	struct dirstack *fdstack = make_stack("fdstack");
-	char *command = (other_files) ? "gtags --find --other | gnusort -t / -k 2"
+	const char *command = (other_files) ? "gtags --find --other | gnusort -t / -k 2"
 					: "gtags --find";
 	struct dirstack *push = make_stack("push");
 	struct dirstack *pop = make_stack("pop");
@@ -606,7 +632,7 @@ makefileindex(file, files)
 	strbuf_reset(sb);
 	strbuf_puts(sb, "\\.(");
 	{
-		char *p = include_file_suffixes;
+		const char *p = include_file_suffixes;
 		int c;
 
 		while ((c = (unsigned char)*p++) != '\0') {
@@ -624,21 +650,20 @@ makefileindex(file, files)
 	/*
 	 * preparations.
 	 */
-	snprintf(indexlink, sizeof(indexlink),
-		(Fflag) ? "../files.%s" : "../mains.%s", normal_suffix);
-
 	if ((FIND = popen(command, "r")) == NULL)
 		die("cannot fork.");
 	if ((FILES = fopen(makepath(distpath, file, NULL), "w")) == NULL)
 		die("cannot open file '%s'.", file);
 
-	fprintf(FILES, "%s\n", html_begin);
-	fprintf(FILES, set_header(title_file_index));
-	fprintf(FILES, "%s\n", body_begin);
-	snprintf(edit, sizeof(edit), "files.%s", normal_suffix); 
-	fprintf(FILES, "%s\n", anchor(Hn(2, title_file_index), edit));
+	fputs_nl(gen_page_begin(title_file_index, TOPDIR), FILES);
+	fputs_nl(body_begin, FILES);
+	fputs(header_begin, FILES);
+	fputs(gen_href_begin(NULL, "files", normal_suffix, NULL), FILES);
+	fputs(title_file_index, FILES);
+	fputs(gen_href_end(), FILES);
+	fputs_nl(header_end, FILES);
 	if (!no_order_list)
-		fprintf(FILES, "%s\n", list_begin);
+		fputs_nl(list_begin, FILES);
 	STDOUT = FILES;
 
 	FILEMAP = NULL;
@@ -655,7 +680,7 @@ makefileindex(file, files)
 			if (!other_files)
 				continue;
 			_++;
-			if (is_binary(_))
+			if (test("b", _))
 				continue;
 			notsource = 1;
 		}
@@ -670,23 +695,29 @@ makefileindex(file, files)
 			(void)shift_stack(pop);
 		}
 		if (count_stack(push) || count_stack(pop)) {
-			char *parent, *path;
+			const char *parent, *path, *suffix;
 
 			while (count_stack(pop)) {
 				(void)pop_stack(dirstack);
-				parent = count_stack(dirstack) ? path2url(join_stack(dirstack)) : indexlink;
+				if (count_stack(dirstack)) {
+					parent = path2fid(join_stack(dirstack));
+					suffix = HTML;
+				} else {
+					parent = indexlink;
+					suffix = normal_suffix;
+				}
 				if (no_order_list)
-					fprintf(STDOUT, "%s\n", br);
+					fputs_nl(br, STDOUT);
 				else
-					fprintf(STDOUT, "%s\n", list_end);
-				fprintf(STDOUT, "<A HREF=%s TITLE='Parent Directory'>", parent);
+					fputs_nl(list_end, STDOUT);
+				fputs(gen_href_begin_with_title(NULL, parent, suffix, NULL, "Parent Directory"), STDOUT);
 				if (icon_list)
-					fprintf(STDOUT, "<IMG SRC=../icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+					fputs(gen_image(PARENT, back_icon, ".."), STDOUT);
 				else
-					fprintf(STDOUT, "[..]");
-				fprintf(STDOUT, "</A>\n");
-				fprintf(STDOUT, "%s\n", body_end);
-				fprintf(STDOUT, "%s\n", html_end);
+					fputs("[..]", STDOUT);
+				fputs_nl(gen_href_end(), STDOUT);
+				fputs_nl(body_end, STDOUT);
+				fputs_nl(gen_page_end(), STDOUT);
 				path = pop_stack(fdstack);	
 				close_file_queue(path);
 				file_count++;
@@ -695,71 +726,74 @@ makefileindex(file, files)
 				pop_stack(pop);
 			}
 			while (count_stack(push)) {
-				char cur[MAXPATHLEN];
-				char *last;
-
-				parent = count_stack(dirstack) ? path2url(join_stack(dirstack)) : indexlink;
+				char cur[MAXPATHLEN], tmp[MAXPATHLEN];
+				const char *last;
+				if (count_stack(dirstack)) {
+					parent = path2fid(join_stack(dirstack));
+					suffix = HTML;
+				} else {
+					parent = indexlink;
+					suffix = normal_suffix;
+				}
 				push_stack(dirstack, shift_stack(push));
 				path = join_stack(dirstack);
-				snprintf(cur, sizeof(cur), "%s/files/%s", distpath, path2url(path));
+				snprintf(cur, sizeof(cur), "%s/files/%s.%s", distpath, path2fid(path), HTML);
 				last = (full_path) ? path : top_stack(dirstack);
 
 				strbuf_reset(sb);
 				if (!no_order_list)
-					strbuf_puts(sb, list_item);
-
-				strbuf_sprintf(sb, "<A HREF=%s%s TITLE='%s/'>",
-					count_stack(dirstack) == 1 ? "files/" : "",
-					path2url(path),
-					path);
+					strbuf_puts(sb, item_begin);
+				snprintf(tmp, sizeof(tmp), "%s/", path);
+				strbuf_puts(sb, gen_href_begin_with_title(count_stack(dirstack) == 1 ? "files" : NULL, path2fid(path), HTML, NULL, tmp));
 				if (icon_list) {
-					strbuf_sprintf(sb, "<IMG SRC=%sicons/%s.%s ALT=[%s/] HSPACE=3 %s>",
-						count_stack(dirstack) == 1 ? "" : "../", dir_icon, icon_suffix, path, icon_spec);
+					strbuf_puts(sb, gen_image(count_stack(dirstack) == 1 ? CURRENT : PARENT, dir_icon, tmp));
+					strbuf_puts(sb, quote_space);
 				}
-				strbuf_sprintf(sb, "%s/</A>\n", last);
-				if (no_order_list)
+				strbuf_sprintf(sb, "%s/%s", last, gen_href_end());
+				if (!no_order_list)
+					strbuf_puts(sb, item_end);
+				else
 					strbuf_puts(sb, br);
+				strbuf_putc(sb, '\n');
 				if (count_stack(dirstack) == 1)
 					strbuf_puts(files, strbuf_value(sb));
 				else
-					fprintf(STDOUT, "%s", strbuf_value(sb));
+					fputs(strbuf_value(sb), STDOUT);
 				op = open_file_queue(cur);
 				STDOUT = op;
 				push_stack(fdstack, cur);
-				fprintf(STDOUT, "%s\n", html_begin);
 				strbuf_reset(sb);
 				strbuf_puts(sb, path);
 				strbuf_putc(sb, '/');
-				fprintf(STDOUT, set_header(strbuf_value(sb)));
-				fprintf(STDOUT, "%s\n", body_begin);
-				fprintf(STDOUT, "<H2>");
-				fprintf(STDOUT, "<A HREF=%s>root</A>/", indexlink);
+				fputs_nl(gen_page_begin(strbuf_value(sb), SUBDIR), STDOUT);
+				fputs_nl(body_begin, STDOUT);
+				fprintf(STDOUT, "%s%sroot%s/", header_begin, gen_href_begin(NULL, indexlink, normal_suffix, NULL), gen_href_end());
 				{
 					struct dirstack *p = make_stack("tmp");
-					char *s;
+					const char *s;
 					int anchor;
 
 					for (s = bottom_stack(dirstack); s; s = next_stack(dirstack, s)) {
 						push_stack(p, s);
 						anchor = count_stack(p) < count_stack(dirstack) ? 1 : 0;
 						if (anchor)
-							fprintf(STDOUT, "<A HREF=%s>", path2url(join_stack(p)));
-						fprintf(STDOUT, s);
+							fputs(gen_href_begin(NULL, path2fid(join_stack(p)), HTML, NULL), STDOUT);
+						fputs(s, STDOUT);
 						if (anchor)
-							fprintf(STDOUT, "</A>");
-						fprintf(STDOUT, "/");
+							fputs(gen_href_end(), STDOUT);
+						fputc('/', STDOUT);
 					}
 					delete_stack(p);
 				}
-				fprintf(STDOUT, "</H2>\n");
-				fprintf(STDOUT, "<A HREF=%s TITLE='Parent Directory'>", parent);
+				fputs_nl(header_end, STDOUT);
+				fputs(gen_href_begin_with_title(NULL, parent, suffix, NULL, "Parent Directory"), STDOUT);
 				if (icon_list)
-					fprintf(STDOUT, "<IMG SRC=../icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+					fputs(gen_image(PARENT, back_icon, ".."), STDOUT);
 				else
-					fprintf(STDOUT, "[..]");
-				fprintf(STDOUT, "</A>\n");
+					fputs("[..]", STDOUT);
+				fputs_nl(gen_href_end(), STDOUT);
 				if (!no_order_list)
-					fprintf(STDOUT, "%s\n", list_begin);
+					fputs_nl(list_begin, STDOUT);
 				else
 					fprintf(STDOUT, "%s%s\n", br, br);
 			}
@@ -775,22 +809,26 @@ makefileindex(file, files)
 		if (regexec(&is_include_file, _, 0, 0, 0) == 0)
 			put_inc(fname, _, count);
 		strbuf_reset(sb);
-		if (!no_order_list) {
-			strbuf_puts(sb, list_item);
-			strbuf_putc(sb, '\n');
+		if (!no_order_list)
+			strbuf_puts(sb, item_begin);
+
+		{
+			char tmp[1024];
+			const char *file, *suffix = NULL, *dir = NULL;
+
+			if (notsource && dynamic) {
+				if (!(*action == '/' || count_stack(dirstack) == 0))
+					dir = "..";
+				snprintf(tmp, sizeof(tmp), "%s?pattern=%s%stype=source",
+					action, encode(_), quote_amp);
+				file = tmp;
+			} else {
+				dir = count_stack(dirstack) ? upperdir(SRCS) : SRCS;
+				file = path2fid(_);
+				suffix = HTML;
+			}
+			strbuf_puts(sb, gen_href_begin_with_title_target(dir, file, suffix, NULL, _, target));
 		}
-		strbuf_puts(sb, "<A HREF=");
-		if (notsource && dynamic) {
-			if (!(*action == '/' || count_stack(dirstack) == 0))
-				strbuf_puts(sb, "../");
-			strbuf_sprintf(sb, "%s?pattern=%s&type=source",
-				action, encode(_));
-		} else {
-			if (count_stack(dirstack))
-				strbuf_puts(sb, "../");
-			strbuf_sprintf(sb, "%s/%s", SRCS, path2url(_));
-		}
-		strbuf_sprintf(sb, " TARGET=%s TITLE='%s'>", target, _);
 		if (icon_list) {
 			const char *lang, *suffix, *text_icon;
 
@@ -801,53 +839,59 @@ makefileindex(file, files)
 				text_icon = c_icon;
 			else
 				text_icon = file_icon;
-			strbuf_puts(sb, "<IMG SRC=");
-			if (count_stack(dirstack))
-				strbuf_puts(sb, "../");
-			strbuf_sprintf(sb, "icons/%s.%s ALT=[%s] HSPACE=3 %s>",
-				text_icon, icon_suffix, _, icon_spec);
+			strbuf_puts(sb, gen_image(count_stack(dirstack) == 0 ? CURRENT : PARENT, text_icon, _));
+			strbuf_puts(sb, quote_space);
 		}
 		if (full_path) {
 			strbuf_puts(sb, _);
 		} else {
-			char *last = locatestring(_, "/", MATCH_LAST);
+			const char *last = locatestring(_, "/", MATCH_LAST);
 			if (last)
 				last++;
 			else
 				last = _;
 			strbuf_puts(sb, last);
 		}
-		strbuf_puts(sb, "</A>\n");
-		if (no_order_list)
-			strbuf_sprintf(sb, "%s\n", br);
+		strbuf_puts(sb, gen_href_end());
+		if (!no_order_list)
+			strbuf_puts(sb, item_end);
+		else
+			strbuf_puts(sb, br);
+		strbuf_putc(sb, '\n');
 		if (map_file)
-			fprintf(FILEMAP, "%s\t%s/%s\n", _, SRCS, path2url(_));
+			fprintf(FILEMAP, "%s\t%s/%s.%s\n", _, SRCS, path2fid(_), HTML);
 		if (count_stack(dirstack) == 0)
 			strbuf_puts(files, strbuf_value(sb));
 		else
-			fprintf(STDOUT, "%s", strbuf_value(sb));
+			fputs(strbuf_value(sb), STDOUT);
 	}
 	if (map_file)
 		fclose(FILEMAP);
 	if (pclose(FIND) != 0)
 		die("cannot traverse directory.(%s)", command);
 	while (count_stack(dirstack) > 0) {
-		char *parent;
+		const char *parent, *suffix;
 
 		pop_stack(dirstack);
-		parent = (count_stack(dirstack) > 0) ? path2url(join_stack(dirstack)) : indexlink;
+		if (count_stack(dirstack) > 0) {
+			parent = path2fid(join_stack(dirstack));
+			suffix = HTML;
+		} else {
+			parent = indexlink;
+			suffix = normal_suffix;
+		}
 		if (no_order_list)
-			fprintf(STDOUT, "%s\n", br);
+			fputs_nl(br, STDOUT);
 		else
-			fprintf(STDOUT, "%s\n", list_end);
-		fprintf(STDOUT, "<A HREF=%s TITLE='Parent Directory'>", parent);
+			fputs_nl(list_end, STDOUT);
+		fputs(gen_href_begin_with_title(NULL, parent, suffix, NULL, "Parent Directory"), STDOUT);
 		if (icon_list)
-			fprintf(STDOUT, "<IMG SRC=../icons/%s.%s ALT='[..]' %s>", back_icon, icon_suffix, icon_spec);
+			fputs(gen_image(PARENT, back_icon, ".."), STDOUT);
 		else
-			fprintf(STDOUT, "[..]");
-		fprintf(STDOUT, "</A>\n");
-		fprintf(STDOUT, "%s\n", body_end);
-		fprintf(STDOUT, "%s\n", html_end);
+			fputs("[..]", STDOUT);
+		fputs_nl(gen_href_end(), STDOUT);
+		fputs_nl(body_end, STDOUT);
+		fputs_nl(gen_page_end(), STDOUT);
 		close_file_queue(pop_stack(fdstack));
 		file_count++;
 		if (count_stack(fdstack) > 0)
@@ -855,11 +899,11 @@ makefileindex(file, files)
 	}
 	fputs(strbuf_value(files), FILES);
 	if (no_order_list)
-		fprintf(FILES, "%s\n", br);
+		fputs_nl(br, FILES);
 	else
-		fprintf(FILES, "%s\n", list_end);
-	fprintf(FILES, "%s\n", body_end);
-	fprintf(FILES, "%s\n", html_end);
+		fputs_nl(list_end, FILES);
+	fputs_nl(body_end, FILES);
+	fputs_nl(gen_page_end(), FILES);
 	fclose(FILES);
 	file_count++;
 
@@ -873,14 +917,14 @@ makefileindex(file, files)
 	return count;
 }
 void
-makeincludeindex()
+makeincludeindex(void)
 {
 	FILE *PIPE;
 	STRBUF *input = strbuf_open(0);
-	char *command;
-	char *_;
+	char *ctags_x;
 	struct data *inc;
 	char *target = (Fflag) ? "mains" : "_top";
+	const char *command = "global -gnx \"^[ \\t]*(#[ \\t]*(import|include)|include[ \\t]*\\()\"";
 
 	/*
 	 * Pick up include pattern.
@@ -888,31 +932,24 @@ makeincludeindex()
 	 * C: #include "xxx.h"
 	 * PHP: include("xxx.inc.php");
 	 */
-	command = "global -gnx \"^[ \\t]*(#[ \\t]*(import|include)|include[ \\t]*\\()\"";
 	if ((PIPE = popen(command, "r")) == NULL)
 		die("cannot fork.");
 	strbuf_reset(input);
-	while ((_ = strbuf_fgets(input, PIPE, STRBUF_NOCRLF)) != NULL) {
+	while ((ctags_x = strbuf_fgets(input, PIPE, STRBUF_NOCRLF)) != NULL) {
 		SPLIT ptable;
-                char *nouse, *lno, *filename, *image;
-		char *last, buf[MAXBUFLEN];
+		char buf[MAXBUFLEN];
 		int is_php = 0;
-		const char *lang, *suffix;
+		const char *last, *lang, *suffix;
 
-		if (split(_, 4, &ptable) < 4) {
+		if (split(ctags_x, 4, &ptable) < 4) {
 			recover(&ptable);
 			die("too small number of parts in makefileindex().");
 		}
-		nouse = ptable.part[0].start;
-		lno = ptable.part[1].start;
-		filename = ptable.part[2].start;
-		image = ptable.part[3].start;
-
-		if ((suffix = locatestring(filename, ".", MATCH_LAST)) != NULL
+		if ((suffix = locatestring(ptable.part[PART_PATH].start, ".", MATCH_LAST)) != NULL
 		    && (lang = decide_lang(suffix)) != NULL
 		    && strcmp(lang, "php") == 0)
 			is_php = 1;
-		last = extract_lastname(image, is_php);
+		last = extract_lastname(ptable.part[PART_LINE].start, is_php);
 		if (last == NULL || get_inc(last) == NULL)
 			continue;
 		recover(&ptable);
@@ -920,11 +957,12 @@ makeincludeindex()
 		 * s/^[^ \t]+/$last/;
 		 */
 		{
-			char *p, *q = buf;
+			const char *p;
+			char *q = buf;
 
 			for (p = last; *p; p++)
 				*q++ = *p;
-			for (p = _; *p && *p != ' ' && *p != '\t'; p++)
+			for (p = ctags_x; *p && *p != ' ' && *p != '\t'; p++)
 				;
 			for (; *p; p++)
 				*q++ = *p;
@@ -936,7 +974,7 @@ makeincludeindex()
 		die("terminated abnormally.");
 
 	for (inc = first_inc(); inc; inc = next_inc()) {
-		char *last = inc->name;
+		const char *last = inc->name;
 		int no = inc->id;
 		struct data *data;
 		FILE *INCLUDE;
@@ -946,20 +984,22 @@ makeincludeindex()
 
 			snprintf(path, sizeof(path), "%s/%s/%d.%s", distpath, INCS, no, HTML);
 			INCLUDE = open_file_queue(path);
-			fprintf(INCLUDE, "%s\n", html_begin);
-			fprintf(INCLUDE, set_header(last));
-			fprintf(INCLUDE, "%s\n", body_begin);
-			fprintf(INCLUDE, "%s\n", verbatim_begin);
+			fputs_nl(gen_page_begin(last, SUBDIR), INCLUDE);
+			fputs_nl(body_begin, INCLUDE);
+			fputs_nl(verbatim_begin, INCLUDE);
 			{
-				char *filename = strbuf_value(inc->contents);
+				const char *filename = strbuf_value(inc->contents);
 				int count = inc->count;
 
-				for (; count; filename += strlen(filename) + 1, count--)
-					fprintf(INCLUDE, "<A HREF=../%s/%s TARGET=%s>%s</A>\n", SRCS, path2url(filename), target, filename);
+				for (; count; filename += strlen(filename) + 1, count--) {
+					fputs(gen_href_begin_with_title_target(upperdir(SRCS), path2fid(filename), HTML, NULL, NULL, target), INCLUDE);
+					fputs(filename, INCLUDE);
+					fputs_nl(gen_href_end(), INCLUDE);
+				}
 			}
-			fprintf(INCLUDE, "%s\n", verbatim_end);
-			fprintf(INCLUDE, "%s\n", body_end);
-			fprintf(INCLUDE, "%s\n", html_end);
+			fputs_nl(verbatim_end, INCLUDE);
+			fputs_nl(body_end, INCLUDE);
+			fputs_nl(gen_page_end(), INCLUDE);
 			close_file_queue(path);
 			file_count++;
 			/*
@@ -973,17 +1013,13 @@ makeincludeindex()
 			continue;
 		if (data->count == 1) {
 			SPLIT ptable;
-			char *nouse, *lno, *filename;
 			char buf[1024];
 
 			if (split(strbuf_value(data->contents), 4, &ptable) < 4) {
 				recover(&ptable);
 				die("too small number of parts in makefileindex().");
 			}
-			nouse = ptable.part[0].start;
-			lno = ptable.part[1].start;
-			filename = ptable.part[2].start;
-			snprintf(buf, sizeof(buf), "%s %s", lno, filename);
+			snprintf(buf, sizeof(buf), "%s %s", ptable.part[PART_LNO].start, ptable.part[PART_PATH].start);
 			recover(&ptable);
 			strbuf_reset(data->contents);
 			strbuf_puts(data->contents, buf);
@@ -992,20 +1028,19 @@ makeincludeindex()
 
 			snprintf(path, sizeof(path), "%s/%s/%d.%s", distpath, INCREFS, no, HTML);
 			INCLUDE = open_file_queue(path);
-			fprintf(INCLUDE, "%s\n", html_begin);
-			fprintf(INCLUDE, set_header(last));
-			fprintf(INCLUDE, "%s\n", body_begin);
-			fprintf(INCLUDE, "%s\n", gen_list_begin());
+			fputs_nl(gen_page_begin(last, SUBDIR), INCLUDE);
+			fputs_nl(body_begin, INCLUDE);
+			fputs_nl(gen_list_begin(), INCLUDE);
 			{
-				char *line = strbuf_value(data->contents);
+				const char *line = strbuf_value(data->contents);
 				int count = data->count;
 
 				for (; count; line += strlen(line) + 1, count--)
-					fprintf(INCLUDE, "%s\n", gen_list_body("../S", line));
+					fputs_nl(gen_list_body(upperdir(SRCS), line), INCLUDE);
 			}
-			fprintf(INCLUDE, "%s\n", gen_list_end());
-			fprintf(INCLUDE, "%s\n", body_end);
-			fprintf(INCLUDE, "%s\n", html_end);
+			fputs_nl(gen_list_end(), INCLUDE);
+			fputs_nl(body_end, INCLUDE);
+			fputs_nl(gen_page_end(), INCLUDE);
 			close_file_queue(path);
 			file_count++;
 			/*
