@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,10 +50,10 @@
 #include "path2url.h"
 #include "const.h"
 
-void src2html(char *, char *, int);
+void src2html(const char *, const char *, int);
 int makedupindex(void);
-int makedefineindex(char *, int, STRBUF *);
-int makefileindex(char *, STRBUF *);
+int makedefineindex(const char *, int, STRBUF *);
+int makefileindex(const char *, STRBUF *);
 void makeincludeindex();
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
@@ -65,24 +65,24 @@ void makeincludeindex();
  * Global data.
  */
 int w32 = W32;				/* Windows32 environment	*/
-char *www = "http://www.gnu.org/software/global/";
-char *include_header;
+const char *www = "http://www.gnu.org/software/global/";
 int file_count = 0;
 int sep = '/';
-char *save_config;
-char *save_argv;
+const char *save_config;
+const char *save_argv;
 
 char cwdpath[MAXPATHLEN];
 char dbpath[MAXPATHLEN];
 char distpath[MAXPATHLEN];
 char gtagsconf[MAXPATHLEN];
+char datadir[MAXPATHLEN];
 
 char sort_path[MAXFILLEN];
 char gtags_path[MAXFILLEN];
 char global_path[MAXFILLEN];
 char findcom[MAXFILLEN];
-char *null_device = NULL_DEVICE;
-char *tmpdir = "/tmp";
+const char *null_device = NULL_DEVICE;
+const char *tmpdir = "/tmp";
 
 /*
  * options
@@ -112,27 +112,31 @@ int no_order_list;			/* 1: doesn't use order list	*/
 int other_files;			/* 1: list other files		*/
 int enable_grep;			/* 1: enable grep		*/
 int enable_idutils;			/* 1: enable idutils		*/
+int enable_xhtml;			/* 1: enable XHTML		*/
 
-char *action_value;
-char *id_value;
-char *cgidir;
-char *main_func = "main";
-char *style_sheet;
-char *cvsweb_url;
-char *cvsweb_cvsroot;
-char *gtagslabel;
-char *title;
+const char *action_value;
+const char *id_value;
+const char *cgidir;
+const char *main_func = "main";
+const char *style_sheet;
+const char *cvsweb_url;
+const char *cvsweb_cvsroot;
+const char *gtagslabel;
+const char *title;
+const char *xhtml_version = "1.0";
+const char *insert_header;		/* --insert-header=<file>	*/
+const char *insert_footer;		/* --insert-footer=<file>	*/
 
 /*
  * Constant values.
  */
-char *title_define_index = "DEFINITIONS";
-char *title_file_index = "FILES";
-char *title_included_from = "INCLUDED FROM";
+const char *title_define_index = "DEFINITIONS";
+const char *title_file_index = "FILES";
+const char *title_included_from = "INCLUDED FROM";
 /*
  * Function header items.
  */
-char *anchor_label[] = {
+const char *anchor_label[] = {
 	"&lt;",
 	"&gt;",
 	"^",
@@ -142,7 +146,7 @@ char *anchor_label[] = {
 	"index",
 	"help"
 };
-char *anchor_icons[] = {
+const char *anchor_icons[] = {
 	"left",
 	"right",
 	"first",
@@ -152,7 +156,7 @@ char *anchor_icons[] = {
 	"index",
 	"help"
 };
-char *anchor_comment[] = {
+const char *anchor_comment[] = {
 	"previous",
 	"next",
 	"first",
@@ -162,7 +166,7 @@ char *anchor_comment[] = {
 	"index",
 	"help"
 };
-char *anchor_msg[] = {
+const char *anchor_msg[] = {
 	"Previous definition.",
 	"Next definition.",
 	"First definition in this file.",
@@ -172,10 +176,10 @@ char *anchor_msg[] = {
 	"Return to index page.",
 	"You are seeing now."
 };
-char *back_icon = "back";
-char *dir_icon  = "dir";
-char *c_icon = "c";
-char *file_icon = "text";
+const char *back_icon = "back";
+const char *dir_icon  = "dir";
+const char *c_icon = "c";
+const char *file_icon = "text";
 
 /*
  * Configuration parameters.
@@ -185,26 +189,26 @@ int tabs = 8;				/* tab skip			*/
 char stabs[8];				/* tab skip (string)		*/
 int full_path = 0;			/* file index format		*/
 int map_file = 1;			/* 1: create MAP file		*/
-char *icon_list = NULL;			/* use icon list		*/
-char *icon_suffix = "png";		/* icon suffix (jpg, png etc)	*/
-char *icon_spec = "BORDER=0 ALIGN=top";	/* parameter in IMG tag		*/
-char *prolog_script = NULL;		/* include script at first	*/
-char *epilog_script = NULL;		/* include script at last	*/
+const char *icon_list = NULL;		/* use icon list		*/
+const char *icon_suffix = "png";	/* icon suffix (jpg, png etc)	*/
+const char *icon_spec = "border='0' align='top'";/* parameter in IMG tag*/
+const char *prolog_script = NULL;	/* include script at first	*/
+const char *epilog_script = NULL;	/* include script at last	*/
 int show_position = 0;			/* show current position	*/
 int table_list = 0;			/* tag list using table tag	*/
 int colorize_warned_line = 0;		/* colorize warned line		*/
-char *script_alias = "/cgi-bin";	/* script alias of WWW server	*/
-char *gzipped_suffix = "ghtml";		/* suffix of gzipped html file	*/
-char *normal_suffix = "html";		/* suffix of normal html file	*/
-char *HTML;
-char *action = "cgi-bin/global.cgi";	/* default action		*/
-char *saction;				/* safe action			*/
-char *id = NULL;			/* id (default non)		*/
+const char *script_alias = "/cgi-bin";	/* script alias of WWW server	*/
+const char *gzipped_suffix = "ghtml";	/* suffix of gzipped html file	*/
+const char *normal_suffix = "html";	/* suffix of normal html file	*/
+const char *HTML;
+const char *action = "cgi-bin/global.cgi";/* default action		*/
+const char *saction;			/* safe action			*/
+const char *id = NULL;			/* id (default non)		*/
 int cgi = 1;				/* 1: make cgi-bin/		*/
 int definition_header=NO_HEADER;	/* (NO|BEFORE|RIGHT|AFTER)_HEADER */
-char *htags_options = NULL;
-char *include_file_suffixes = "h,hxx,hpp,H,inc.php";
-static char *langmap = DEFAULTLANGMAP;
+const char *htags_options = NULL;
+const char *include_file_suffixes = "h,hxx,hpp,H,inc.php";
+static const char *langmap = DEFAULTLANGMAP;
 
 static struct option const long_options[] = {
         {"alphabet", no_argument, NULL, 'a'},
@@ -222,6 +226,7 @@ static struct option const long_options[] = {
         {"title", required_argument, NULL, 't'},
         {"verbose", no_argument, NULL, 'v'},
         {"warning", no_argument, NULL, 'w'},
+        {"xhtml", no_argument, NULL, 'x'},
 
         /* long name only */
         {"action", required_argument, NULL, 1},
@@ -234,6 +239,8 @@ static struct option const long_options[] = {
         {"id", required_argument, NULL, 1},
         {"nocgi", no_argument, NULL, 1},
         {"no-map-file", no_argument, &no_map_file, 1},
+        {"insert-header", required_argument, NULL, 1},
+        {"insert-footer", required_argument, NULL, 1},
         {"statistics", no_argument, &statistics, 1},
         {"style-sheet", required_argument, NULL, 1},
         {"version", no_argument, &show_version, 1},
@@ -242,14 +249,14 @@ static struct option const long_options[] = {
 };
 
 static void
-usage()
+usage(void)
 {
         if (!qflag)
                 fputs(usage_const, stderr);
         exit(2);
 }
 static void
-help()
+help(void)
 {
         fputs(usage_const, stdout);
         fputs(help_const, stdout);
@@ -259,7 +266,7 @@ help()
  * Htags catch signal even if the parent ignore it.
  */
 void
-clean()
+clean(void)
 {
 	unload_gpath();
 	cache_close();
@@ -283,7 +290,7 @@ suddenly(signo)
  * Setup signal hander.
  */
 static void
-signal_setup()
+signal_setup(void)
 {
         signal(SIGINT, suddenly);
         signal(SIGTERM, suddenly);
@@ -303,8 +310,8 @@ signal_setup()
  */
 static void
 generate_file(dist, file)
-	char *dist;
-	char *file;
+	const char *dist;
+	const char *file;
 {
 	regex_t preg;
 	regmatch_t pmatch[2];
@@ -313,18 +320,31 @@ generate_file(dist, file)
 	char *_;
 	int i;
         struct map {
-                char *name;
-                char *value;
+                const char *name;
+                const char *value;
         } tab[] = {
-                {"@html_begin@", html_begin},
-                {"@html_end@", html_end},
+		/* dynamic initialization */
+                {"@page_begin@", NULL},
+                {"@page_end@", NULL},
+                {"@lineno_anchor@", NULL},
+
+		/* static initialization */
                 {"@body_begin@", body_begin},
                 {"@body_end@", body_end},
+                {"@title_begin@", title_begin},
+                {"@title_end@", title_end},
+                {"@error_begin@", error_begin},
+                {"@error_end@", error_end},
+                {"@message_begin@", message_begin},
+                {"@message_end@", message_end},
+                {"@verbatim_begin@", verbatim_begin},
+                {"@verbatim_end@", verbatim_end},
                 {"@global_path@", global_path},
                 {"@gtags_path@", gtags_path},
                 {"@normal_suffix@", normal_suffix},
                 {"@tabs@", stabs},
                 {"@hr@", hr},
+                {"@br@", br},
                 {"@HTML@", HTML},
                 {"@action@", action},
                 {"@null_device@", null_device},
@@ -333,6 +353,9 @@ generate_file(dist, file)
         };
 	int tabsize = sizeof(tab) / sizeof(struct map);
 
+	tab[0].value = gen_page_begin("Result", SUBDIR);
+	tab[1].value = gen_page_end();
+	tab[2].value = gen_name_string("L$.");
 	/*
 	 * construct regular expression.
 	 */
@@ -346,15 +369,13 @@ generate_file(dist, file)
 	if (regcomp(&preg, strbuf_value(sb), REG_EXTENDED) != 0)
 		die("cannot compile regular expression.");
 	/*
-	 * construct skelton file name in the system datadir directory.
+	 * construct skeleton file name in the system datadir directory.
 	 */
 	strbuf_reset(sb);
-	if (!getconfs("datadir", sb))
-		die("cannot get datadir directory name.");
-	strbuf_sprintf(sb, "/gtags/%s.tmpl", file);
+	strbuf_sprintf(sb, "%s/gtags/%s.tmpl", datadir, file);
 	ip = fopen(strbuf_value(sb), "r");
 	if (!ip)
-		die("skelton file '%s' not found.", strbuf_value(sb));
+		die("skeleton file '%s' not found.", strbuf_value(sb));
 	op = fopen(makepath(dist, file, NULL), "w");
 	if (!op)
 		die("cannot create file '%s'.", file);
@@ -363,11 +384,11 @@ generate_file(dist, file)
 	 * Read templete file and evaluate macros.
 	 */
 	while ((_ = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
-		char *p;
+		const char *p;
 
 		/* Pick up macro name */
 		for (p = _; !regexec(&preg, p, 2, pmatch, 0); p += pmatch[0].rm_eo) {
-			char *start = p + pmatch[0].rm_so;
+			const char *start = p + pmatch[0].rm_so;
 			int length = pmatch[0].rm_eo - pmatch[0].rm_so;
 
 			/* print before macro */
@@ -380,19 +401,20 @@ generate_file(dist, file)
 				die("something wrong.");
 			/* print macro value */
 			if (i < tabsize) {
-				char *q;
+				const char *q;
 				/*
 				 * Double quote should be quoted using '\\'.
 				 */
 				for (q = tab[i].value; *q; q++) {
 					if (*q == '"')
 						fputc('\\', op);
+					else if (*q == '\n')
+						fputc('\\', op);
 					fputc(*q, op);
 				}
 			}
 		}
-		fputs(p, op);
-		fputc('\n', op);
+		fputs_nl(p, op);
 	}
 	fclose(op);
 	fclose(ip);
@@ -406,8 +428,8 @@ generate_file(dist, file)
  */
 static void
 makeprogram(cgidir, file)
-	char *cgidir;
-	char *file;
+	const char *cgidir;
+	const char *file;
 {
 	generate_file(cgidir, file);
 }
@@ -416,9 +438,9 @@ makeprogram(cgidir, file)
  */
 static void
 makebless(file)
-	char *file;
+	const char *file;
 {
-	char *save = action;
+	const char *save = action;
 	action = saction;
 	generate_file(distpath, file);
 	action = save;
@@ -431,17 +453,17 @@ makebless(file)
  */
 static void
 makeghtml(cgidir, file)
-	char *cgidir;
-	char *file;
+	const char *cgidir;
+	const char *file;
 {
 	FILE *op;
 
 	op = fopen(makepath(cgidir, file, NULL), "w");
 	if (!op)
 		die("cannot make unzip script.");
-	fprintf(op, "#!/bin/sh\n");
-	fprintf(op, "echo \"content-type: text/html\"\n");
-	fprintf(op, "echo\n");
+	fputs_nl("#!/bin/sh", op);
+	fputs_nl("echo \"content-type: text/html\"", op);
+	fputs_nl("echo", op);
 	fprintf(op, "gzip -S %s -d -c \"$PATH_TRANSLATED\"\n", HTML);
         fclose(op);
 }
@@ -450,20 +472,20 @@ makeghtml(cgidir, file)
  */
 static void
 makerebuild(file)
-	char *file;
+	const char *file;
 {
 	FILE *op;
 
 	op = fopen(makepath(distpath, file, NULL), "w");
 	if (!op)
 		die("cannot make rebuild script.");
-	fprintf(op, "#!/bin/sh\n");
-	fprintf(op, "#\n");
-	fprintf(op, "# rebuild.sh: rebuild hypertext with the previous context.\n");
-	fprintf(op, "#\n");
-	fprintf(op, "# Usage:\n");
-	fprintf(op, "#\t%% sh rebuild.sh\n");
-	fprintf(op, "#\n");
+	fputs_nl("#!/bin/sh", op);
+	fputs_nl("#", op);
+	fputs_nl("# rebuild.sh: rebuild hypertext with the previous context.", op);
+	fputs_nl("#", op);
+	fputs_nl("# Usage:", op);
+	fputs_nl("#\t% sh rebuild.sh", op);
+	fputs_nl("#", op);
 	fprintf(op, "cd %s && GTAGSCONF='%s' htags%s\n", cwdpath, save_config, save_argv);
         fclose(op);
 }
@@ -472,26 +494,27 @@ makerebuild(file)
  */
 static void
 makehelp(file)
-	char *file;
+	const char *file;
 {
-	char **label = icon_list ? anchor_comment : anchor_label;
-	char **icons = anchor_icons;
-	char **msg   = anchor_msg;
+	const char **label = icon_list ? anchor_comment : anchor_label;
+	const char **icons = anchor_icons;
+	const char **msg   = anchor_msg;
 	int n, last = 7;
 	FILE *op;
 
 	op = fopen(makepath(distpath, file, NULL), "w");
 	if (!op)
 		die("cannot make help file.");
-	fprintf(op, "%s\n", html_begin);
-	fprintf(op, "%s", set_header("HELP"));
-	fprintf(op, "%s\n", body_begin);
-	fprintf(op, "<H2>Usage of Links</H2>\n");
-	fprintf(op, "%s/* ", verbatim_begin);
+	fputs_nl(gen_page_begin("HELP", TOPDIR), op);
+	fputs_nl(body_begin, op);
+	fputs(header_begin, op);
+	fputs("Usage of Links", op);
+	fputs_nl(header_end, op);
+	fputs(verbatim_begin, op);
+	fputs("/* ", op);
 	for (n = 0; n <= last; n++) {
 		if (icon_list) {
-			fprintf(op, "<IMG SRC=icons/%s.%s ALT=[%s] %s>",
-					icons[n], icon_suffix, label[n], icon_spec);
+			fputs(gen_image(CURRENT, icons[n], label[n]), op);
 			if (n < last)
 				fputc(' ', op);
 		} else {
@@ -500,25 +523,32 @@ makehelp(file)
 	}
 	if (show_position)
 		fprintf(op, "[+line file]");
-	fprintf(op, " */%s\n", verbatim_end);
-	fprintf(op, "<DL>\n");
+	fputs(" */", op);
+	fputs_nl(verbatim_end, op);
+	fputs_nl(define_list_begin, op);
 	for (n = 0; n <= last; n++) {
-		fprintf(op, "<DT>");
+		fputs(define_term_begin, op);
 		if (icon_list) {
-			fprintf(op, "<IMG SRC=icons/%s.%s ALT=[%s] %s>",
-					icons[n], icon_suffix, label[n], icon_spec);
+			fputs(gen_image(CURRENT, icons[n], label[n]), op);
 		} else {
 			fprintf(op, "[%s]", label[n]);
 		}
-		fprintf(op, "<DD>%s\n", msg[n]);
+		fputs(define_term_end, op);
+		fputs(define_desc_begin, op);
+		fputs(msg[n], op);
+		fputs_nl(define_desc_end, op);
 	}
 	if (show_position) {
-		fprintf(op, "<DT>[+line file]");
-		fprintf(op, "<DD>Current position (line number and file name).\n");
+		fputs(define_term_begin, op);
+		fputs("[+line file]", op);
+		fputs(define_term_end, op);
+		fputs(define_desc_begin, op);
+		fputs("Current position (line number and file name).", op);
+		fputs_nl(define_desc_end, op);
 	}
-	fprintf(op, "</DL>\n");
-	fprintf(op, "%s\n", body_end);
-	fprintf(op, "%s\n", html_end);
+	fputs_nl(define_list_end, op);
+	fputs_nl(body_end, op);
+	fputs_nl(gen_page_end(), op);
 	fclose(op);
 	file_count++;
 }
@@ -532,63 +562,59 @@ makehelp(file)
  */
 static char *
 makesearchpart(action, id, target)
-	char *action;
-	char *id;
-	char *target;
+	const char *action;
+	const char *id;
+	const char *target;
 {
-	STRBUF *sb = strbuf_open(0);
+	STATIC_STRBUF(sb);
 
-	if (Fflag) {
-		strbuf_puts(sb, "<A HREF=search.");
-		strbuf_puts(sb, normal_suffix);
-		strbuf_puts(sb, "><H2>SEARCH</H2></A>\n");
-	} else {
-		strbuf_puts(sb, "<H2>SEARCH</H2>\n");
+	strbuf_clear(sb);
+	strbuf_puts(sb, header_begin);
+	if (Fflag)
+		strbuf_puts(sb, gen_href_begin(NULL, "search", normal_suffix, NULL));
+	strbuf_puts(sb, "SEARCH");
+	if (Fflag)
+		strbuf_puts(sb, gen_href_end());
+	strbuf_puts_nl(sb, header_end);
+	if (!target) {
+		strbuf_puts(sb, "Please input object name and select [Search]. POSIX's regular expression is allowed.");
+		strbuf_puts_nl(sb, br);
 	}
-	if (!target)
-		strbuf_puts(sb, "Please input object name and select [Search]. POSIX's regular expression is allowed.<P>\n"); 
-	strbuf_puts(sb, "<FORM METHOD=GET ACTION=");
-	strbuf_puts(sb, action);
-	if (target) {
-		strbuf_puts(sb, " TARGET=");
-		strbuf_puts(sb, target);
-	}
-	strbuf_puts(sb, ">\n");
-	strbuf_puts(sb, "<INPUT NAME=pattern>\n");
-	strbuf_puts(sb, "<INPUT TYPE=hidden NAME=id VALUE=");
-	if (id)
-		strbuf_puts(sb, id);
-	strbuf_puts(sb, ">\n");
-	strbuf_puts(sb, "<INPUT TYPE=submit VALUE=Search>\n");
-	strbuf_puts(sb, "<INPUT TYPE=reset VALUE=Reset><BR>\n");
-	strbuf_puts(sb, "<INPUT TYPE=radio NAME=type VALUE=definition CHECKED TITLE='Retrieve the definition place of the specified symbol.'>");
-	strbuf_puts(sb, target ? "Def" : "Definition");
-	strbuf_puts(sb, "\n<INPUT TYPE=radio NAME=type VALUE=reference TITLE='Retrieve the reference place of the specified symbol.'>");
-	strbuf_puts(sb, target ? "Ref" : "Reference");
+	strbuf_puts_nl(sb, gen_form_begin(target));
+	strbuf_puts_nl(sb, gen_input("pattern", NULL, NULL));
+	if (id == NULL)
+		id = "";
+	strbuf_puts_nl(sb, gen_input("id", id, "hidden"));
+	strbuf_puts_nl(sb, gen_input(NULL, "Search", "submit"));
+	strbuf_puts(sb, gen_input(NULL, "Reset", "reset"));
+	strbuf_puts_nl(sb, br);
+	strbuf_puts(sb, gen_input_radio("type", "definition", 1, "Retrieve the definition place of the specified symbol."));
+	strbuf_puts_nl(sb, target ? "Def" : "Definition");
+	strbuf_puts(sb, gen_input_radio("type", "reference", 0, "Retrieve the reference place of the specified symbol."));
+	strbuf_puts_nl(sb, target ? "Ref" : "Reference");
 	if (test("f", makepath(dbpath, dbname(GSYMS), NULL))) {
-		strbuf_puts(sb, "\n<INPUT TYPE=radio NAME=type VALUE=symbol TITLE='Retrieve the place of the specified symbol is used.'>");
-		strbuf_puts(sb, target ? "Sym" : "Other symbol");
+		strbuf_puts(sb, gen_input_radio("type", "symbol", 0, "Retrieve the place of the specified symbol is used."));
+		strbuf_puts_nl(sb, target ? "Sym" : "Other symbol");
 	}
-	strbuf_puts(sb, "\n<INPUT TYPE=radio NAME=type VALUE=path TITLE='Look for path name which matches to the specified pattern.'>");
+	strbuf_puts(sb, gen_input_radio("type", "path", 0, "Look for path name which matches to the specified pattern."));
 	strbuf_puts(sb, target ? "Path" : "Path name");
+	strbuf_puts_nl(sb, br);
 	if (enable_grep) {
-		strbuf_puts(sb, "\n<INPUT TYPE=radio NAME=type VALUE=grep TITLE='Retrieve lines which matches to the specified pattern.'>");
-		strbuf_puts(sb, target ? "Grep" : "Grep pattern");
+		strbuf_puts(sb, gen_input_radio("type", "grep", 0, "Retrieve lines which matches to the specified pattern."));
+		strbuf_puts_nl(sb, target ? "Grep" : "Grep pattern");
 	}
 	if (enable_idutils && test("f", makepath(dbpath, "ID", NULL))) {
-		strbuf_puts(sb, "\n<INPUT TYPE=radio NAME=type VALUE=idutils TITLE='Retrieve lines which matches to the specified pattern using idutils(1).'>");
-		strbuf_puts(sb, target ? "Id" : "Id pattern");
+		strbuf_puts(sb, gen_input_radio("type", "idutils", 0, "Retrieve lines which matches to the specified pattern using idutils(1)."));
+		strbuf_puts_nl(sb, target ? "Id" : "Id pattern");
 	}
-	strbuf_puts(sb, "<BR>\n<INPUT TYPE=checkbox NAME=icase VALUE=1 TITLE='Ignore case distinctions in the pattern.'>");
-	strbuf_puts(sb, target ? "Icase" : "Ignore case");
+	strbuf_puts(sb, gen_input_checkbox("icase", "1", "Ignore case distinctions in the pattern."));
+	strbuf_puts_nl(sb, target ? "Icase" : "Ignore case");
 	if (other_files) {
-		strbuf_puts(sb, "\n<INPUT TYPE=checkbox NAME=other VALUE=1 TITLE='Files other than the source code are also retrieved.'>");
-		strbuf_puts(sb, target ? "Other" : "Other files");
+		strbuf_puts(sb, gen_input_checkbox("other", "1", "Files other than the source code are also retrieved."));
+		strbuf_puts_nl(sb, target ? "Other" : "Other files");
 	}
-	strbuf_puts(sb, "\n</FORM>\n");
-
+	strbuf_puts_nl(sb, gen_form_end());
 	return strbuf_value(sb);
-	/* doesn't close string buffer */
 }
 /*
  * makeindex: make index file
@@ -599,9 +625,9 @@ makesearchpart(action, id, target)
  */
 static void
 makeindex(file, title, index)
-	char *file;
-	char *title;
-	char *index;
+	const char *file;
+	const char *title;
+	const char *index;
 {
 	FILE *op;
 
@@ -609,37 +635,39 @@ makeindex(file, title, index)
 	if (!op)
 		die("cannot make file '%s'.", file);
 	if (Fflag) {
-		fprintf(op, "%s\n", html_begin);
-		fprintf(op, "<HEAD>\n<TITLE>%s</TITLE>\n", title);
-		fprintf(op, "%s", meta_record());
-		if (style_sheet)
-			fprintf(op, "%s", style_sheet);
-		fprintf(op, "</HEAD>\n");
-		fprintf(op, "<FRAMESET COLS='200,*'>\n");
+		fputs_nl(gen_frameset_page_begin(title), op);
+		fputs_nl(gen_frameset_begin("cols='200,*'"), op);
 		if (fflag) {
-			fprintf(op, "<FRAMESET ROWS='33%%,33%%,*'>\n");
-			fprintf(op, "<FRAME NAME=search SRC=search.%s>\n", normal_suffix);
+			fputs_nl(gen_frameset_begin("rows='33%,33%,*'"), op);
+			fputs_nl(gen_frame("search", makepath(NULL, "search", normal_suffix)), op);
 		} else {
-			fprintf(op, "<FRAMESET ROWS='50%%,*'>\n");
+			fputs_nl(gen_frameset_begin("rows='50%,*'"), op);
 		}
-		fprintf(op, "<FRAME NAME=defines SRC=defines.%s>\n", normal_suffix);
-		fprintf(op, "<FRAME NAME=files SRC=files.%s>\n", normal_suffix);
-		fprintf(op, "</FRAMESET>\n");
-		fprintf(op, "<FRAME NAME=mains SRC=mains.%s>\n", normal_suffix);
-		fprintf(op, "<NOFRAMES>\n");
-		fprintf(op, "%s\n", body_begin);
+		/*
+		 * id='xxx' for XHTML
+		 * name='xxx' for HTML
+		 */
+		fputs_nl(gen_frame("defines", makepath(NULL, "defines", normal_suffix)), op);
+		fputs_nl(gen_frame("files", makepath(NULL, "files", normal_suffix)), op);
+		fputs_nl(gen_frameset_end(), op);
+		fputs_nl(gen_frame("mains", makepath(NULL, "mains", normal_suffix)), op);
+		fputs_nl(noframes_begin, op);
+		fputs_nl(body_begin, op);
 		fputs(index, op);
-		fprintf(op, "%s\n", body_end);
-		fprintf(op, "</NOFRAMES>\n");
-		fprintf(op, "</FRAMESET>\n");
-		fprintf(op, "%s\n", html_end);
+		fputs_nl(body_end, op);
+		fputs_nl(noframes_end, op);
+		fputs_nl(gen_frameset_end(), op);
+		fputs_nl(gen_page_end(), op);
 	} else {
-		fprintf(op, "%s\n", html_begin);
-		fprintf(op, "%s", set_header(title));
-		fprintf(op, "%s\n", body_begin);
+		fputs_nl(gen_page_begin(title, TOPDIR), op);
+		fputs_nl(body_begin, op);
+		if (insert_header)
+			fputs(gen_insert_header(TOPDIR), op);
 		fputs(index, op);
-		fprintf(op, "%s\n", body_end);
-		fprintf(op, "%s\n", html_end);
+		if (insert_footer)
+			fputs(gen_insert_footer(TOPDIR), op);
+		fputs_nl(body_end, op);
+		fputs_nl(gen_page_end(), op);
 	}
 	fclose(op);
 	file_count++;
@@ -652,20 +680,23 @@ makeindex(file, title, index)
  */
 static void
 makemainindex(file, index)
-	char *file;
-	char *index;
+	const char *file;
+	const char *index;
 {
 	FILE *op;
 
 	op = fopen(makepath(distpath, file, NULL), "w");
 	if (!op)
 		die("cannot make file '%s'.", file);
-	fprintf(op, "%s\n", html_begin);
-	fprintf(op, "%s", set_header(title));
-	fprintf(op, "%s\n", body_begin);
+	fputs_nl(gen_page_begin(title, TOPDIR), op);
+	fputs_nl(body_begin, op);
+	if (insert_header)
+		fputs(gen_insert_header(TOPDIR), op);
 	fputs(index, op);
-	fprintf(op, "%s\n", body_end);
-	fprintf(op, "%s\n", html_end);
+	if (insert_footer)
+		fputs(gen_insert_footer(TOPDIR), op);
+	fputs_nl(body_end, op);
+	fputs_nl(gen_page_end(), op);
 	fclose(op);
 	file_count++;
 }
@@ -676,19 +707,18 @@ makemainindex(file, index)
  */
 static void
 makesearchindex(file)
-	char *file;
+	const char *file;
 {
 	FILE *op;
 
 	op = fopen(makepath(distpath, file, NULL), "w");
 	if (!op)
 		die("cannot create file '%s'.", file);
-	fprintf(op, "%s\n", html_begin);
-	fprintf(op, "%s", set_header("SEARCH"));
-	fprintf(op, "%s\n", body_begin);
+	fputs_nl(gen_page_begin("SEARCH", TOPDIR), op);
+	fputs_nl(body_begin, op);
 	fputs(makesearchpart(action, id, "mains"), op);
-	fprintf(op, "%s\n", body_end);
-	fprintf(op, "%s\n", html_end);
+	fputs_nl(body_end, op);
+	fputs_nl(gen_page_end(), op);
 	fclose(op);
 	file_count++;
 }
@@ -697,26 +727,26 @@ makesearchindex(file)
  */
 static void
 makehtaccess(file)
-	char *file;
+	const char *file;
 {
 	FILE *op;
 
 	op = fopen(makepath(distpath, file, NULL), "w");
 	if (!op)
 		die("cannot make .htaccess skeleton file.");
-	fprintf(op, "#\n");
-	fprintf(op, "# Skelton file for .htaccess -- This file was generated by htags(1).\n");
-	fprintf(op, "#\n");
-	fprintf(op, "# Htags have made gzipped hypertext because you specified -c option.\n");
-	fprintf(op, "# If your browser doesn't decompress gzipped hypertext, you will need to\n");
-	fprintf(op, "# setup your http server to treat this hypertext as gzipped files first.\n");
-	fprintf(op, "# There are many way to do it, but one of the method is to put .htaccess\n");
-	fprintf(op, "# file in 'HTML' directory.\n");
-	fprintf(op, "#\n");
-	fprintf(op, "# Please rewrite '/cgi-bin/ghtml.cgi' to the true value in your web site.\n");
-	fprintf(op, "#\n");
+	fputs_nl("#", op);
+	fputs_nl("# Skeleton file for .htaccess -- This file was generated by htags(1).", op);
+	fputs_nl("#", op);
+	fputs_nl("# Htags have made gzipped hypertext because you specified -c option.", op);
+	fputs_nl("# If your browser doesn't decompress gzipped hypertext, you will need to", op);
+	fputs_nl("# setup your http server to treat this hypertext as gzipped files first.", op);
+	fputs_nl("# There are many way to do it, but one of the method is to put .htaccess", op);
+	fputs_nl("# file in 'HTML' directory.", op);
+	fputs_nl("#", op);
+	fputs_nl("# Please rewrite '/cgi-bin/ghtml.cgi' to the true value in your web site.", op);
+	fputs_nl("#", op);
 	fprintf(op, "AddHandler htags-gzipped-html %s\n", gzipped_suffix);
-	fprintf(op, "Action htags-gzipped-html /cgi-bin/ghtml.cgi\n");
+	fputs_nl("Action htags-gzipped-html /cgi-bin/ghtml.cgi", op);
 	fclose(op);
 }
 /*
@@ -729,10 +759,9 @@ makehtml(total)
 	int total;
 {
 	FILE *ip;
-	char *_, *p;
+	const char *_;
 	int count = 0;
-	char command[MAXFILLEN];
-	char path[MAXPATHLEN];
+	char command[MAXFILLEN], path[MAXPATHLEN];
 	STRBUF *sb = strbuf_open(0);
 
 	if (other_files && !dynamic)
@@ -744,12 +773,13 @@ makehtml(total)
 		die("cannot execute command '%s'.", command);
 	while ((_ = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
 		int notsource = 0;
+		const char *p;
 
 		if (*_ == ' ') {
 			if (!other_files)
 				continue;
 			_++;
-			if (is_binary(_)) {
+			if (test("b", _)) {
 				if (wflag)
 					warning("'%s' is binary file. (skipped)", _);
 				continue;
@@ -760,8 +790,8 @@ makehtml(total)
 		_ += 2;
 		p = _;
 		message(" [%d/%d] converting %s", count, total, p);
-		p = path2url(p);
-		snprintf(path, sizeof(path), "%s/%s/%s", distpath, SRCS, p);
+		p = path2fid(p);
+		snprintf(path, sizeof(path), "%s/%s/%s.%s", distpath, SRCS, p, HTML);
 		src2html(_, path, notsource);
 	}
 	if (pclose(ip) != 0)
@@ -773,8 +803,8 @@ makehtml(total)
  */
 static void
 copyfile(from, to)
-	char *from;
-	char *to;
+	const char *from;
+	const char *to;
 {
 	int ip, op, size;
 	char buf[8192];
@@ -801,8 +831,8 @@ copyfile(from, to)
 static void
 duplicatefile(file, from, to)
 	const char *file;
-	char *from;
-	char *to;
+	const char *from;
+	const char *to;
 {
 	char from_path[MAXPATHLEN];
 	char to_path[MAXPATHLEN];
@@ -826,84 +856,92 @@ duplicatefile(file, from, to)
  */
 static char *
 makecommonpart(title, defines, files)
-	char *title;
-	char *defines;
-	char *files;
+	const char *title;
+	const char *defines;
+	const char *files;
 {
 	FILE *ip;
 	STRBUF *sb = strbuf_open(0);
 	STRBUF *ib = strbuf_open(0);
 	char command[MAXFILLEN];
-	char *_;
+	const char *_;
 
-	if (include_header) {
-		strbuf_puts(sb, include_header);
-		strbuf_sprintf(sb, "\n%s\n", hr);
-	}
 	strbuf_puts(sb, title_begin);
 	strbuf_puts(sb, title);
-	strbuf_puts(sb, title_end);
-	strbuf_putc(sb, '\n');
-	strbuf_puts(sb, "<DIV ALIGN=right>\n");
-	strbuf_sprintf(sb, "Last updated %s<BR>\n", now());
-	strbuf_sprintf(sb, "This hypertext was generated by <A HREF=%s TARGET=_top TITLE='Go to the GLOBAL project page.'>GLOBAL-%s</A>.<BR>\n", www, get_version());
-	strbuf_puts(sb, "</DIV>\n");
-	strbuf_sprintf(sb, "%s\n", hr);
+	strbuf_puts_nl(sb, title_end);
+	strbuf_puts_nl(sb, gen_div_begin("right"));
+	strbuf_sprintf(sb, "Last updated %s%s\n", now(), br);
+	strbuf_sprintf(sb, "This hypertext was generated by %sGLOBAL-%s%s.%s\n",
+		gen_href_begin_with_title_target(NULL, www, NULL, NULL, "Go to the GLOBAL project page.", "_top"),
+		get_version(),
+		gen_href_end(),
+		br);
+	strbuf_puts_nl(sb, gen_div_end());
+	strbuf_puts_nl(sb, hr);
 	if (caution) {
-		strbuf_puts(sb, "<CENTER>\n");
-		strbuf_puts(sb, "<BLOCKQUOTE>\n");
-		strbuf_puts(sb, "<FONT SIZE=+2 COLOR=red>CAUTION</FONT><BR>\n");
+		strbuf_puts_nl(sb, caution_begin);
+		strbuf_sprintf(sb, "<font size='+2' color='red'>CAUTION</font>%s\n", br);
 		strbuf_sprintf(sb, "This hypertext consist of %d files.\n", file_count);
-		strbuf_puts(sb, "Please don't download whole hypertext using hypertext copy tools.\n");
-		strbuf_puts(sb, "Our network cannot afford such traffic.\n");
-		strbuf_puts(sb, "Instead, you can generate same thing in your computer using\n");
-		strbuf_sprintf(sb, "<A HREF=%s TARGET=_top>GLOBAL source code tag system</A>.\n", www);
-		strbuf_puts(sb, "Thank you. \n");
-		strbuf_puts(sb, "</BLOCKQUOTE>\n");
-		strbuf_puts(sb, "</CENTER>\n");
+		strbuf_puts_nl(sb, "Please don't download whole hypertext using hypertext copy tools.");
+		strbuf_puts_nl(sb, "Our network cannot afford such traffic.");
+		strbuf_puts_nl(sb, "Instead, you can generate same thing in your computer using");
+		strbuf_puts(sb, gen_href_begin_with_title_target(NULL, www, NULL, NULL, NULL, "_top"));
+		strbuf_puts(sb, "GLOBAL source code tag system");
+		strbuf_puts_nl(sb, gen_href_end());
+		strbuf_puts_nl(sb, "Thank you.");
+		strbuf_puts_nl(sb, caution_end);
 		strbuf_sprintf(sb, "\n%s\n", hr);
 	}
 	if (fflag) {
 		strbuf_puts(sb, makesearchpart(action, id, NULL));
-		strbuf_sprintf(sb, "%s\n", hr);
+		strbuf_puts_nl(sb, hr);
 	}
-	strbuf_puts(sb, "<H2>MAINS</H2>\n");
+	strbuf_sprintf(sb, "%sMAINS%s\n", header_begin, header_end);
 
 	snprintf(command, sizeof(command), "%s -nx %s | gnusort -k 1,1 -k 3,3 -k 2,2n", global_path, main_func);
         ip = popen(command, "r");
         if (!ip)
                 die("cannot execute command '%s'.", command);
-	strbuf_puts(sb, gen_list_begin());
-	strbuf_putc(sb, '\n');
+	strbuf_puts_nl(sb, gen_list_begin());
 	while ((_ = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
-		strbuf_puts(sb, gen_list_body(SRCS, _));
-		strbuf_putc(sb, '\n');
+		strbuf_puts_nl(sb, gen_list_body(SRCS, _));
 	}
-	strbuf_puts(sb, gen_list_end());
-	strbuf_putc(sb, '\n');
+	strbuf_puts_nl(sb, gen_list_end());
 	if (pclose(ip) != 0)
 		die("cannot execute command '%s'.", command);
-	strbuf_sprintf(sb, "%s\n", hr);
+	strbuf_puts_nl(sb, hr);
 	if (aflag && !Fflag) {
-		strbuf_sprintf(sb, "<H2>%s</H2>\n", title_define_index);
+		strbuf_puts(sb, header_begin);
+		strbuf_puts(sb, title_define_index);
+		strbuf_puts_nl(sb, header_end);
 		strbuf_puts(sb, defines);
 	} else {
-		strbuf_sprintf(sb, "<H2><A HREF=defines.%s>%s</A></H2>\n", normal_suffix, title_define_index);
+		strbuf_puts(sb, header_begin);
+		strbuf_puts(sb, gen_href_begin(NULL, "defines", normal_suffix, NULL));
+		strbuf_puts(sb, title_define_index);
+		strbuf_puts(sb, gen_href_end());
+		strbuf_puts_nl(sb, header_end);
 	}
-	strbuf_sprintf(sb, "%s\n", hr);
+	strbuf_puts_nl(sb, hr);
 	if (Fflag) {
-		strbuf_sprintf(sb, "<H2><A HREF=files.%s>%s</A></H2>\n", normal_suffix, title_file_index);
+		strbuf_puts(sb, header_begin);
+		strbuf_puts(sb, gen_href_begin(NULL, "files", normal_suffix, NULL));
+		strbuf_puts(sb, title_file_index);
+		strbuf_puts(sb, gen_href_end());
+		strbuf_puts_nl(sb, header_end);
 	} else {
-		strbuf_sprintf(sb, "<H2>%s</H2>\n", title_file_index);
+		strbuf_puts(sb, header_begin);
+		strbuf_puts(sb, title_file_index);
+		strbuf_puts_nl(sb, header_end);
 		if (!no_order_list)
-			strbuf_puts(sb, "<OL>\n");
+			strbuf_puts_nl(sb, list_begin);
 		strbuf_puts(sb, files);
 		if (!no_order_list) {
-			strbuf_puts(sb, "</OL>\n");
+			strbuf_puts_nl(sb, list_end);
 		} else {
-			strbuf_puts(sb, "<BR>\n");
+			strbuf_puts_nl(sb, br);
 		}
-		strbuf_sprintf(sb, "%s\n", hr);
+		strbuf_puts_nl(sb, hr);
 	}
 	strbuf_close(ib);
 
@@ -914,9 +952,9 @@ makecommonpart(title, defines, files)
  * basic check.
  */
 static void
-basic_check()
+basic_check(void)
 {
-	char *p;
+	const char *p;
 
 	/*
 	 * COMMAND EXISTENCE CHECK
@@ -1003,6 +1041,10 @@ configuration(argc, argv)
 	/*
 	 * Config variables.
 	 */
+	strbuf_reset(sb);
+	if (!getconfs("datadir", sb))
+		die("cannot get datadir directory name.");
+	strlimcpy(datadir, strbuf_value(sb), sizeof(datadir));
 	if (getconfn("ncol", &n)) {
 		if (n < 1 || n > 10)
 			warning("parameter 'ncol' ignored becase the value (=%d) is too large or too small.", n);
@@ -1102,11 +1144,11 @@ configuration(argc, argv)
 		p = strdup(strbuf_value(sb));
 		if (p == NULL)
 			die("short of memory.");
-		script_alias = p;
 		/* remove the last '/' */
-		p = script_alias + strlen(script_alias) - 1;
-		if (*p == '/')
-			*p = '\0';
+		q = p + strlen(p) - 1;
+		if (*q == '/')
+			*q = '\0';
+		script_alias = p;
 	}
 	if (getconfb("symbols"))	/* for backward compatibility */
 		symbol = 1;
@@ -1272,6 +1314,13 @@ configuration(argc, argv)
 			die("short of memory.");
 		langmap = p;
 	}
+	strbuf_reset(sb);
+	if (getconfs("xhtml_version", sb)) {
+		p = strdup(strbuf_value(sb));
+		if (p == NULL)
+			die("short of memory.");
+		xhtml_version = p;
+	}
 	/* insert htags_options into the head of ARGSV array. */
 	strbuf_reset(sb);
 	if (getconfs("htags_options", sb)) {
@@ -1295,7 +1344,7 @@ save_environment(argc, argv)
 	STRBUF *save_c = strbuf_open(0);
 	STRBUF *save_a = strbuf_open(0);
 	int i;
-	char *p;
+	const char *p;
 	FILE *ip;
 
 	/*
@@ -1363,10 +1412,10 @@ append_options(argc, argv)
 {
 
 	STRBUF *sb = strbuf_open(0);
-	char *p, *opt = strdup(htags_options);
+	const char *p, *opt = strdup(htags_options);
 	int count = 1;
 	int quote = 0;
-	char **newargv;
+	const char **newargv;
 	int i = 0, j = 1;
 
 	if (!opt)
@@ -1395,7 +1444,7 @@ append_options(argc, argv)
 			strbuf_putc(sb, *p);
 		}
 	}
-	newargv = (char **)malloc(sizeof(char *) * (*argc + count + 1));
+	newargv = (const char **)malloc(sizeof(char *) * (*argc + count + 1));
 	if (!newargv)
 		die("Short of memory.");
 	newargv[i++] = argv[0];
@@ -1407,7 +1456,7 @@ append_options(argc, argv)
 	while (j < *argc)
 		newargv[i++] = argv[j++];
 	newargv[i] = NULL;
-	argv = newargv;
+	argv = (char **)newargv;
 	*argc = i;
 #ifdef DEBUG
 	for (i = 0; i < *argc; i++)
@@ -1422,10 +1471,10 @@ main(argc, argv)
         int argc;
         char *argv[];
 {
-	char *path, *av = NULL;
+	const char *path, *av = NULL;
 	int func_total, file_total;
         char arg_dbpath[MAXPATHLEN];
-	char *index = NULL;
+	const char *index = NULL;
 	int optchar;
         int option_index = 0;
 	time_t start_time, end_time, start_all_time, end_all_time,
@@ -1434,7 +1483,6 @@ main(argc, argv)
 
 	arg_dbpath[0] = 0;
 	basic_check();
-	setup_html();
 	configuration(argc, argv);
 	setup_langmap(langmap);
 	save_environment(argc, argv);
@@ -1445,7 +1493,7 @@ main(argc, argv)
 	if (htags_options)
 		argv = append_options(&argc, argv);
 
-	while ((optchar = getopt_long(argc, argv, "acd:DfFgm:noqsS:t:vw", long_options, &option_index)) != EOF) {
+	while ((optchar = getopt_long(argc, argv, "acd:DfFgm:noqsS:t:vwx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
                 case 0:
                 case 1:
@@ -1465,6 +1513,10 @@ main(argc, argv)
 				;	/* --gtagslabel is estimated only once. */
 			else if (!strcmp("style-sheet", long_options[option_index].name))
                                 style_sheet = optarg;
+			else if (!strcmp("insert-header", long_options[option_index].name))
+				insert_header = optarg;
+			else if (!strcmp("insert-footer", long_options[option_index].name))
+				insert_footer = optarg;
                         break;
                 case 'a':
                         aflag++;
@@ -1516,11 +1568,18 @@ main(argc, argv)
                 case 'w':
                         wflag++;
                         break;
+		case 'x':
+			enable_xhtml = 1;
+                        break;
                 default:
                         usage();
                         break;
 		}
 	}
+	if (insert_header && !test("fr", insert_header))
+		die("page header file '%s' not found.", insert_header);
+	if (insert_footer && !test("fr", insert_footer))
+		die("page footer file '%s' not found.", insert_footer);
 	if (no_map_file)
 		map_file = 0;
         argc -= optind;
@@ -1534,6 +1593,13 @@ main(argc, argv)
                 setquiet();
 		vflag = 0;
 	}
+	/*
+	 * If the --xhtml option is specified then all HTML tags which
+	 * are defined in configuration file are ignored. Instead, you can
+	 * customize XHTML tag using style sheet (See 'style.css').
+	 */
+	if (enable_xhtml)
+		setup_xhtml();
 	/*
 	 * If copy_files is true then htags copy tag files instead of linking.
 	 * Since Windows 32 environment doesn't have link system call
@@ -1689,6 +1755,7 @@ main(argc, argv)
 	 *       HTML/S/                 ... source files (9)
 	 *       HTML/I/                 ... include file index (9)
 	 *       HTML/rebuild.sh         ... rebuild script (10)
+	 *       HTML/style.css          ... style sheet (11)
 	 *------------------------------------------------------------------
 	 */
 	/* for clean up */
@@ -1888,7 +1955,15 @@ main(argc, argv)
 	makerebuild("rebuild.sh");
 	if (chmod(makepath(distpath, "rebuild.sh", NULL), 0640) < 0)
 		die("cannot chmod rebuild script.");
-
+	/*
+	 * (11) style sheet file (style.css)
+	 */
+	if (enable_xhtml) {
+		char src[MAXPATHLEN], dst[MAXPATHLEN];
+		snprintf(src, sizeof(src), "%s/gtags/style.css", datadir);
+		snprintf(dst, sizeof(dst), "%s/style.css", distpath);
+		copyfile(src, dst);
+	}
 	end_all_time = time(NULL);
 	message("[%s] Done.", now());
 	T_all = end_all_time - start_all_time;
@@ -1901,7 +1976,7 @@ main(argc, argv)
 			message(" gzipped files. (Please see 'HTML/.htaccess')\n");
 		}
 		if (fflag || dynamic) {
-			char *path = (*action == '/') ? makepath("DOCUMENT_ROOT", action, NULL) : makepath("HTML", action, NULL);
+			const char *path = (*action == '/') ? makepath("DOCUMENT_ROOT", action, NULL) : makepath("HTML", action, NULL);
 
 			message(" You need to setup http server so that %s", path);
 			message(" is executed as a CGI script. (DOCUMENT_ROOT means WWW server's data root.)\n");
