@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004 Tama Communications Corporation
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,7 +50,7 @@
  *		using output procedures in this module.
  */
 struct lang_entry {
-	char *lang_name;
+	const char *lang_name;
 	void (*init_proc)(FILE *);		/* initializing procedure */
 	int (*exec_proc)(void);			/* executing procedure */
 };
@@ -64,23 +64,23 @@ void php_parser_init(FILE *);
 void asm_parser_init(FILE *);
 
 /* executing procedures */
-int clex(void);
-int cpplex(void);
-int javalex(void);
-int phplex(void);
-int asmlex(void);
+int c_lex(void);
+int cpp_lex(void);
+int java_lex(void);
+int php_lex(void);
+int asm_lex(void);
 
 /*
  * The first entry is default language.
  */
 struct lang_entry lang_switch[] = {
 	/* lang_name	init_proc 		exec_proc */
-	{"c",		c_parser_init,		clex},		/* DEFAULT */
-	{"yacc",	yacc_parser_init,	clex},
-	{"cpp",		cpp_parser_init,	cpplex},
-	{"java",	java_parser_init,	javalex},
-	{"php",		php_parser_init,	phplex},
-	{"asm",		asm_parser_init,	asmlex}
+	{"c",		c_parser_init,		c_lex},		/* DEFAULT */
+	{"yacc",	yacc_parser_init,	c_lex},
+	{"cpp",		cpp_parser_init,	cpp_lex},
+	{"java",	java_parser_init,	java_lex},
+	{"php",		php_parser_init,	php_lex},
+	{"asm",		asm_parser_init,	asm_lex}
 };
 #define DEFAULT_ENTRY &lang_switch[0]
 
@@ -118,8 +118,8 @@ get_lang_entry(lang)
 static FILE *out;
 static FILE *in;
 
-static STRBUF *outbuf;
-static char *curpfile;
+STATIC_STRBUF(outbuf);
+static const char *curpfile;
 static int warned;
 static int last_lineno;
 
@@ -131,7 +131,7 @@ static int last_lineno;
  */
 static FILE *
 open_input_file(file)
-	char *file;
+	const char *file;
 {
 	char command[MAXFILLEN];
 	FILE *ip;
@@ -151,7 +151,7 @@ static void
 close_input_file(ip)
 	FILE *ip;
 {
-	if (pclose(ip) < 0)
+	if (pclose(ip) != 0)
 		die("command 'gtags --expand -%d' failed.", tabs);
 }
 /*
@@ -162,7 +162,7 @@ close_input_file(ip)
  */
 static FILE *
 open_output_file(file)
-	char *file;
+	const char *file;
 {
 	char command[MAXFILLEN];
 	FILE *op;
@@ -177,10 +177,7 @@ open_output_file(file)
 		if (!op)
 			die("cannot create file '%s'.", file);
 	}
-	if (!outbuf)
-		outbuf = strbuf_open(0);
-	else
-		strbuf_reset(outbuf);
+	strbuf_clear(outbuf);
 	return op;
 }
 /*
@@ -191,7 +188,7 @@ close_output_file(op)
 	FILE *op;
 {
 	if (cflag) {
-		if (pclose(op) < 0)
+		if (pclose(op) != 0)
 			die("command 'gzip -c' failed.");
 	} else
 		fclose(op);
@@ -229,33 +226,28 @@ echos(s)
  *       i)      $path   path name
  *       r)              hypertext file name string
  */
-char *
+const char *
 fill_anchor(root, path)
-	char *root;
-	char *path;
+	const char *root;
+	const char *path;
 {
-	static STRBUF *sb = NULL;
+	STATIC_STRBUF(sb);
 	char buf[MAXBUFLEN], *limit, *p;
 
-	if (sb)
-		strbuf_reset(sb);
-	else
-		sb = strbuf_open(0);
+	strbuf_clear(sb);
 	strlimcpy(buf, path, sizeof(buf));
 	for (p = buf; *p; p++)
 		if (*p == sep)
 			*p = '\0';
 	limit = p;
 
-	strbuf_puts(sb, "<A HREF=");
-	strbuf_puts(sb, root);
-	strbuf_puts(sb, ">root</A>/");
+	strbuf_sprintf(sb, "%sroot%s/", gen_href_begin_simple(root), gen_href_end());
 	{
-		char *next;
+		const char *next;
 
 		for (p = buf; p < limit; p += strlen(p) + 1) {
-			char *path = buf;
-			char *unit = p;
+			const char *path = buf;
+			const char *unit = p;
 
 			next = p + strlen(p) + 1;
 			if (next > limit) {
@@ -264,8 +256,10 @@ fill_anchor(root, path)
 			}
 			if (p > buf)
 				*(p - 1) = sep;
-			strbuf_sprintf(sb, "<A HREF=../files/%s>%s</A>/",
-				path2url(path), unit);
+			strbuf_puts(sb, gen_href_begin("../files", path2fid(path), HTML, NULL));
+			strbuf_puts(sb, unit);
+			strbuf_puts(sb, gen_href_end());
+			strbuf_putc(sb, '/');
 		}
 	}
         return strbuf_value(sb);
@@ -275,53 +269,44 @@ fill_anchor(root, path)
  * link_format: make hypertext from anchor array.
  *
  *	i)	(previous, next, first, last, top, bottom)
+ *		-1: top, -2: bottom, other: line number
  *	r)	HTML
  */
-char *
+const char *
 link_format(ref)
         int ref[A_SIZE];
 {
-	static STRBUF *sb = NULL;
-	char **label = icon_list ? anchor_comment : anchor_label;
-	char **icons = anchor_icons;
+	STATIC_STRBUF(sb);
+	const char **label = icon_list ? anchor_comment : anchor_label;
+	const char **icons = anchor_icons;
 	int i;
 
-	if (sb)
-		strbuf_reset(sb);
-	else
-		sb = strbuf_open(0);
+	strbuf_clear(sb);
 	for (i = 0; i < A_LIMIT; i++) {
 		if (i == A_INDEX) {
-			strbuf_puts(sb, "<A HREF=../mains.");
-			strbuf_puts(sb, normal_suffix);
-			strbuf_putc(sb, '>');
+			strbuf_puts(sb, gen_href_begin("..", "mains", normal_suffix, NULL));
 		} else if (i == A_HELP) {
-			strbuf_puts(sb, "<A HREF=../help.");
-			strbuf_puts(sb, normal_suffix);
-			strbuf_putc(sb, '>');
+			strbuf_puts(sb, gen_href_begin("..", "help", normal_suffix, NULL));
 		} else if (ref[i]) {
-			strbuf_puts(sb, "<A HREF=#");
+			char tmp[32], *key = tmp;
+
 			if (ref[i] == -1)
-				strbuf_puts(sb, "TOP");
+				key = "TOP";
 			else if (ref[i] == -2)
-				strbuf_puts(sb, "BOTTOM");
+				key = "BOTTOM";
 			else
-				strbuf_putn(sb, ref[i]);
-			strbuf_putc(sb, '>');
+				snprintf(tmp, sizeof(tmp), "%d", ref[i]);
+			strbuf_puts(sb, gen_href_begin(NULL, NULL, NULL, key));
 		}
 		if (icon_list) {
-			strbuf_puts(sb, "<IMG SRC=../icons/");
-			if (i != A_INDEX && i != A_HELP && ref[i] == 0)
-				strbuf_puts(sb, "n_");
-			strbuf_puts(sb, icons[i]);
-			strbuf_putc(sb, '.');
-			strbuf_puts(sb, icon_suffix);
-			strbuf_sprintf(sb, " ALT=[%s] %s>", label[i], icon_spec);
+			char tmp[MAXPATHLEN];
+			snprintf(tmp, sizeof(tmp), "%s%s", (i != A_INDEX && i != A_HELP && ref[i] == 0) ? "n_" : "", icons[i]);
+			strbuf_puts(sb, gen_image(PARENT, tmp, label[i]));
 		} else {
 			strbuf_sprintf(sb, "[%s]", label[i]);
 		}
 		if (i == A_INDEX || i == A_HELP || ref[i] != 0)
-			strbuf_puts(sb, "</A>");
+			strbuf_puts(sb, gen_href_end());
 	}
         return strbuf_value(sb);
 }
@@ -331,17 +316,14 @@ link_format(ref)
  *	i)	lineno	line number
  *	r)		guide string
  */
-char *
+const char *
 generate_guide(lineno)
 	int lineno;
 {
-	static STRBUF *sb = NULL;
+	STATIC_STRBUF(sb);
 	int i = 0;
 
-	if (!sb)
-		sb = strbuf_open(0);
-	else
-		strbuf_reset(sb);
+	strbuf_clear(sb);
 	if (definition_header == RIGHT_HEADER)
 		i = 4;
 	else if (nflag)
@@ -366,18 +348,15 @@ generate_guide(lineno)
  *	i)	opt	
  *	r)		tooltip string
  */
-char *
+const char *
 tooltip(type, lno, opt)
 	int type;
 	int lno;
-	char *opt;
+	const char *opt;
 {
-	static STRBUF *sb = NULL;
+	STATIC_STRBUF(sb);
 
-	if (!sb)
-		sb = strbuf_open(0);
-	else
-		strbuf_reset(sb);
+	strbuf_clear(sb);
 	if (lno > 0) {
 		if (type == 'I')
 			strbuf_puts(sb, "Included from");
@@ -424,7 +403,7 @@ put_anchor(name, type, lineno)
 	int type;
 	int lineno;
 {
-	char *line;
+	const char *line;
 	int db;
 
 	if (type == 'R')
@@ -443,42 +422,51 @@ put_anchor(name, type, lineno)
 		}
 		strbuf_puts(outbuf, name);
 	} else {
+		/*
+		 * About cache record format, please see the comment in cache.c.
+		 */
 		if (*line == ' ') {
-			int id = atoi(++line);
-			char *count = locatestring(line, " ", MATCH_FIRST) + 1;
+			char tmp[MAXPATHLEN];
+			const char *id = strmake(++line, " ");
+			const char *count = locatestring(line, " ", MATCH_FIRST) + 1;
+			const char *dir, *file, *suffix = NULL;
 
-			strbuf_puts(outbuf, "<A HREF=");
 			if (dynamic) {
-				char *dir = (*action == '/') ? "" : "../";
-				char *s;
+				const char *s;
 
+				dir = (*action == '/') ? NULL : "..";
 				if (db == GTAGS)
 					s = "definitions";
 				else if (db == GRTAGS)
 					s = "reference";
 				else
 					s = "symbol";
-				strbuf_sprintf(outbuf, "%s%s?pattern=%s&type=%s",
-					dir, action, name, s);
+				snprintf(tmp, sizeof(tmp), "%s?pattern=%s%stype=%s",
+					action, name, quote_amp, s);
+				file = tmp;
 			} else {
-				char *dir;
-
 				if (type == 'R')
-					dir = DEFS;
+					dir = upperdir(DEFS);
 				else if (type == 'Y')
-					dir = SYMS;
+					dir = upperdir(SYMS);
 				else	/* 'D', 'M' or 'T' */
-					dir = REFS;
-				strbuf_sprintf(outbuf, "../%s/%d.%s", dir, id, HTML);
+					dir = upperdir(REFS);
+				file = id;
+				suffix = HTML;
 			}
-			strbuf_sprintf(outbuf, " TITLE=\"%s\">%s</A>", tooltip(type, -1, count), name);
+			strbuf_puts(outbuf, gen_href_begin_with_title(dir, file, suffix, NULL, tooltip(type, -1, count)));
+			strbuf_puts(outbuf, name);
+			strbuf_puts(outbuf, gen_href_end());
 		} else {
-			int lno = atoi(line);
-			char *filename = strmake(locatestring(line, " ", MATCH_FIRST) + 1, " ")
+			char lno[32];
+			const char *filename;
+
+			strlimcpy(lno, strmake(line, " "), sizeof(lno));
+			filename = strmake(locatestring(line, " ", MATCH_FIRST) + 1, " ")
 						+ 2;	/* remove './' */
-			char *url = path2url(filename);
-			strbuf_sprintf(outbuf, "<A HREF=../%s/%s#%d TITLE=\"%s\">%s</A>",
-				SRCS, url, lno, tooltip(type, lno, filename), name);
+			strbuf_puts(outbuf, gen_href_begin_with_title(upperdir(SRCS), path2fid(filename), HTML, lno, tooltip(type, atoi(lno), filename)));
+			strbuf_puts(outbuf, name);
+			strbuf_puts(outbuf, gen_href_end());
 		}
 	}
 }
@@ -491,23 +479,24 @@ put_anchor(name, type, lineno)
 void
 put_include_anchor(inc, path)
 	struct data *inc;
-	char *path;
+	const char *path;
 {
-	strbuf_puts(outbuf, "<A HREF=");
 	if (inc->count == 1)
-		strbuf_puts(outbuf, path2url(strbuf_value(inc->contents)));
-	else
-		strbuf_sprintf(outbuf, "../%s/%d.%s", INCS, inc->id, HTML);
-	strbuf_putc(outbuf, '>');
+		strbuf_puts(outbuf, gen_href_begin(NULL, path2fid(strbuf_value(inc->contents)), HTML, NULL));
+	else {
+		char id[32];
+		snprintf(id, sizeof(id), "%d", inc->id);
+		strbuf_puts(outbuf, gen_href_begin(upperdir(INCS), id, HTML, NULL));
+	}
 	strbuf_puts(outbuf, path);
-	strbuf_puts(outbuf, "</A>");
+	strbuf_puts(outbuf, gen_href_end());
 }
 /*
  * Put a reserved word. (if, while, ...)
  */
 void
 put_reserved_word(word)
-        char *word;
+	const char *word;
 {
 	strbuf_puts(outbuf, reserved_begin);
 	strbuf_puts(outbuf, word);
@@ -518,7 +507,7 @@ put_reserved_word(word)
  */
 void
 put_macro(word)
-        char *word;
+	const char *word;
 {
 	strbuf_puts(outbuf, sharp_begin);
 	strbuf_puts(outbuf, word);
@@ -529,7 +518,7 @@ put_macro(word)
  */
 void
 unknown_preprocessing_directive(word, lineno)
-	char *word;
+	const char *word;
 	int lineno;
 {
 	word = strtrim(word, TRIM_ALL, NULL);
@@ -553,7 +542,7 @@ unexpected_eof(lineno)
  */
 void
 unknown_yacc_directive(word, lineno)
-	char *word;
+	const char *word;
 	int lineno;
 {
 	warning("unknown yacc directive '%s'. [+%d %s]", word, lineno, curpfile);
@@ -565,7 +554,7 @@ unknown_yacc_directive(word, lineno)
  */
 void
 missing_left(word, lineno)
-	char *word;
+	const char *word;
 	int lineno;
 {
 	warning("missing left '%s'. [+%d %s]", word, lineno, curpfile);
@@ -597,7 +586,7 @@ put_char(c)
  */
 void
 put_string(s)
-        char *s;
+        const char *s;
 {
 	for (; *s; s++)
 		put_char(*s);
@@ -607,7 +596,7 @@ put_string(s)
  */
 void
 put_brace(text)
-        char *text;
+        const char *text;
 {
 	strbuf_puts(outbuf, brace_begin);
 	strbuf_puts(outbuf, text);
@@ -618,7 +607,7 @@ put_brace(text)
  * common procedure for line control.
  */
 static char lineno_format[32];
-static char *guide = NULL;
+static const char *guide = NULL;
 
 /*
  * Begin of line processing.
@@ -634,8 +623,7 @@ put_begin_of_line(lineno)
                         guide = NULL;
         }
         if (guide && definition_header == BEFORE_HEADER) {
-                fputs(guide, out);
-                fputc('\n', out);
+                fputs_nl(guide, out);
                 guide = NULL;
         }
 }
@@ -652,7 +640,7 @@ void
 put_end_of_line(lineno)
 	int lineno;
 {
-	fprintf(out, "<A NAME=%d>", lineno);
+	fputs(gen_name_number(lineno), out);
         if (nflag)
                 fprintf(out, lineno_format, lineno);
 	if (warned)
@@ -671,8 +659,7 @@ put_end_of_line(lineno)
 			fputs(guide, out);
 		fputc('\n', out);
 		if (definition_header == AFTER_HEADER) {
-			fputs(guide, out);
-			fputc('\n', out);
+			fputs_nl(guide, out);
 		}
 		guide = NULL;
 	}
@@ -691,8 +678,8 @@ put_end_of_line(lineno)
  */
 void
 src2html(src, html, notsource)
-	char *src;
-	char *html;
+	const char *src;
+	const char *html;
 	int notsource;
 {
 	char indexlink[128];
@@ -715,22 +702,22 @@ src2html(src, html, notsource)
 	if (!notsource) {
 		anchor_load(src);
 	}
-        fprintf(out, "%s\n", html_begin);
-        fputs(set_header(src), out);
-        fprintf(out, "%s\n", body_begin);
+	fputs_nl(gen_page_begin(src, SUBDIR), out);
+	fputs_nl(body_begin, out);
 	/*
          * print the header
          */
-        fputs("<A NAME=TOP><H2>", out);
+	if (insert_header)
+		fputs(gen_insert_header(SUBDIR), out);
+	fputs(gen_name_string("TOP"), out);
+	fputs(header_begin, out);
         fputs(fill_anchor(indexlink, src), out);
 	if (cvsweb_url) {
-		static STRBUF *sb = NULL;
-		char *p;
+		STATIC_STRBUF(sb);
+		const char *p;
 
-		if (sb)
-			strbuf_reset(sb);
-		else
-			sb = strbuf_open(0);
+		strbuf_clear(sb);
+		strbuf_puts(sb, cvsweb_url);
 		for (p = src; *p; p++) {
 			int c = (unsigned char)*p;
 
@@ -739,31 +726,37 @@ src2html(src, html, notsource)
 			else
 				strbuf_sprintf(sb, "%%%02x", c);
 		}
-        	fprintf(out, "%s<A HREF=%s%s", quote_space, cvsweb_url, strbuf_value(sb));
 		if (cvsweb_cvsroot)
-        		fprintf(out, "?cvsroot=%s", cvsweb_cvsroot);
-        	fprintf(out, "><FONT SIZE=-1>[CVS]</FONT></A>\n");
+			strbuf_sprintf(sb, "?cvsroot=%s", cvsweb_cvsroot);
+		fputs(quote_space, out);
+		fputs(gen_href_begin_simple(strbuf_value(sb)), out);
+		fputs(cvslink_begin, out);
+		fputs("[CVS]", out);
+		fputs(cvslink_end, out);
+		fputs_nl(gen_href_end(), out);
 		/* doesn't close string buffer */
 	}
-	fprintf(out, "</H2>\n");
-        fprintf(out, "%s/* ", comment_begin);
+	fputs_nl(header_end, out);
+	fputs(comment_begin, out);
+	fputs("/* ", out);
 
 	fputs(link_format(anchor_getlinks(0)), out);
 	if (show_position)
 		fprintf(out, "%s[+1 %s]%s", position_begin, src, position_end);
-	fprintf(out, " */%s", comment_end);
-	fprintf(out, "\n%s\n", hr);
+	fputs(" */", out);
+	fputs_nl(comment_end, out);
+	fputs_nl(hr, out);
         /*
          * It is not source file.
          */
         if (notsource) {
 		STRBUF *sb = strbuf_open(0);
-		char *_;
+		const char *_;
 
-		fprintf(out, "%s\n", verbatim_begin);
+		fputs_nl(verbatim_begin, out);
 		last_lineno = 0;
 		while ((_ = strbuf_fgets(sb, in, STRBUF_NOCRLF)) != NULL) {
-			fprintf(out, "<A NAME=%d>", ++last_lineno);
+			fputs(gen_name_number(++last_lineno), out);
 			for (; *_; _++) {
 				int c = *_;
 
@@ -778,17 +771,17 @@ src2html(src, html, notsource)
 			}
 			fputc('\n', out);
 		}
-		fprintf(out, "%s\n", verbatim_end);
+		fputs_nl(verbatim_end, out);
 		strbuf_close(sb);
         }
 	/*
 	 * It's source code.
 	 */
 	else {
-		char *basename;
+		const char *basename;
 		struct data *incref;
 		struct anchor *ancref;
-		static STRBUF *define_index = NULL;
+		STATIC_STRBUF(define_index);
 
                 /*
                  * INCLUDED FROM index.
@@ -800,59 +793,72 @@ src2html(src, html, notsource)
 			basename = src;
 		incref = get_included(basename);
 		if (incref) {
-			fputs("<H2><A HREF=", out);
+			char s_id[32];
+			const char *dir, *file, *suffix, *key, *title;
+
+			fputs(header_begin, out);
 			if (incref->count > 1) {
 				char s_count[32];
 
 				snprintf(s_count, sizeof(s_count), "%d", incref->count);
-				fprintf(out, "../%s/%d.%s", INCREFS, incref->id, HTML);
-				fprintf(out, " TITLE='%s'>", tooltip('I', -1, s_count));
+				snprintf(s_id, sizeof(s_id), "%d", incref->id);
+				dir = upperdir(INCREFS);
+				file = s_id;
+				suffix = HTML;
+				key = NULL;
+				title = tooltip('I', -1, s_count);
 			} else {
-				char *lno, *filename, *save;
-				char *p = strbuf_value(incref->contents);
+				const char *p = strbuf_value(incref->contents);
+				const char *lno = strmake(p, " ");
+				const char *filename;
 
-				lno = p;
-				while (*p != ' ')
-					p++;
-				save = p;
-				*p++ = '\0';
-				filename = p;
+				p = locatestring(p, " ", MATCH_FIRST);
+				if (p == NULL)
+					die("internal error.(incref->contents)");
+				filename = p + 1;
 				if (filename[0] == '.' && filename[1] == '/')
 					filename += 2;
-				fprintf(out, "%s#%s", path2url(filename), lno);
-				fprintf(out, " TITLE='%s'>", tooltip('I', atoi(lno), filename));
-				*save = ' ';
+				dir = NULL;
+				file = path2fid(filename);
+				suffix = HTML;
+				key = lno;
+				title = tooltip('I', atoi(lno), filename);
 			}
-			fprintf(out, "%s</A></H2>\n", title_included_from);
-			fprintf(out, "%s\n", hr);
+			fputs(gen_href_begin_with_title(dir, file, suffix, key, title), out);
+			fputs(title_included_from, out);
+			fputs(gen_href_end(), out);
+			fputs_nl(header_end, out);
+			fputs_nl(hr, out);
 		}
 		/*
 		 * DEFINITIONS index.
 		 */
-		if (define_index)
-			strbuf_reset(define_index);
-		else
-			define_index = strbuf_open(0);
+		strbuf_clear(define_index);
 		for (ancref = anchor_first(); ancref; ancref = anchor_next()) {
 			if (ancref->type == 'D') {
-				strbuf_sprintf(define_index, "<LI><A HREF=#%d TITLE=\"%s\">%s</A>\n",
-					ancref->lineno,
-					tooltip('R', ancref->lineno, NULL),
-					gettag(ancref));
+				char tmp[32];
+				snprintf(tmp, sizeof(tmp), "%d", ancref->lineno);
+				strbuf_puts(define_index, item_begin);
+				strbuf_puts(define_index, gen_href_begin_with_title(NULL, NULL, NULL, tmp, tooltip('R', ancref->lineno, NULL)));
+				strbuf_puts(define_index, gettag(ancref));
+				strbuf_puts(define_index, gen_href_end());
+				strbuf_puts_nl(define_index, item_end);
 			}
 		}
 		if (strbuf_getlen(define_index) > 0) {
-			fprintf(out, "<H2>%s</H2>\n", title_define_index);
-			fputs("This source file includes following definitions.\n", out);
-			fputs("<OL>\n", out);
+			fputs(header_begin, out);
+			fputs(title_define_index, out);
+			fputs_nl(header_end, out);
+			fputs_nl("This source file includes following definitions.", out);
+			fputs_nl(list_begin, out);
 			fputs(strbuf_value(define_index), out);
-			fputs("</OL>\n", out);
-			fprintf(out, "%s\n", hr);
+			fputs_nl(list_end, out);
+			fputs_nl(hr, out);
 		}
 		/*
 		 * print source code
 		 */
-		fprintf(out, "%s\n", verbatim_begin);
+		fputs_nl(verbatim_begin, out);
 		{
 			const char *suffix = locatestring(src, ".", MATCH_LAST);
 			const char *lang = NULL;
@@ -879,18 +885,21 @@ src2html(src, html, notsource)
 			while (ent->exec_proc())
 				;
 		}
-        	fprintf(out, "%s\n", verbatim_end);
+		fputs_nl(verbatim_end, out);
 	}
-	fprintf(out, "%s\n", hr);
-	fputs("<A NAME=BOTTOM>\n", out);
-        fprintf(out, "%s/* ", comment_begin);
+	fputs_nl(hr, out);
+	fputs_nl(gen_name_string("BOTTOM"), out);
+	fputs(comment_begin, out);
+	fputs("/* ", out);
 	fputs(link_format(anchor_getlinks(-1)), out);
 	if (show_position)
 		fprintf(out, "%s[+%d %s]%s", position_begin, last_lineno, src, position_end);
-        fprintf(out, " */%s\n", comment_end);
-        fprintf(out, "%s\n", body_end);
-        fprintf(out, "%s\n", html_end);
-
+	fputs(" */", out);
+	fputs_nl(comment_end, out);
+	if (insert_footer)
+		fputs(gen_insert_footer(SUBDIR), out);
+	fputs_nl(body_end, out);
+	fputs_nl(gen_page_end(), out);
 	if (!notsource)
 		anchor_unload();
 	close_output_file(out);
