@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,7 @@ static int init;
 
 /* initialize for isregex() */
 static void
-initialize()
+initialize(void)
 {
 	regexchar['^'] = regexchar['$'] = regexchar['{'] =
 	regexchar['}'] = regexchar['('] = regexchar[')'] =
@@ -48,7 +48,7 @@ initialize()
  */
 int
 isregexchar(c)
-int c;
+	int c;
 {
 	if (!init)
 		initialize();
@@ -62,7 +62,7 @@ int c;
  */
 int
 isregex(s)
-char *s;
+	const char *s;
 {
 	int c;
 
@@ -78,16 +78,13 @@ char *s;
  *
  *	'aaa' => \'\a\a\a\'
  */
-char *
+const char *
 quote_string(s)
-char *s;
+	const char *s;
 {
-	static STRBUF *sb = NULL;
+	STATIC_STRBUF(sb);
 
-	if (sb == NULL)
-		sb = strbuf_open(0);
-	else
-		strbuf_reset(sb);
+	strbuf_clear(sb);
 	for (; *s; s++) {
 		strbuf_putc(sb, '\\');
 		strbuf_putc(sb, *s);
