@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001, 2002 Tama Communications Corporation
+ * Copyright (c) 1998, 1999, 2000, 2001, 2002, 2005
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +37,7 @@
 #include "conf.h"
 #include "die.h"
 #include "env.h"
+#include "langmap.h"
 #include "locatestring.h"
 #include "makepath.h"
 #include "path.h"
@@ -56,7 +57,7 @@ static int allowed_nest_level = 8;
 static int opened;
 
 static void trim(char *);
-static char *readrecord(const char *);
+static const char *readrecord(const char *);
 static void includelabel(STRBUF *, const char *, int);
 
 #ifndef isblank
@@ -111,7 +112,7 @@ trim(l)
  * o append following line.
  * o format check.
  */
-static char *
+static const char *
 readrecord(label)
 	const char *label;
 {
@@ -134,7 +135,7 @@ readrecord(label)
 		}
 		trim(p);
 		for (;;) {
-			char *candidate;
+			const char *candidate;
 			/*
 			 * pick up candidate.
 			 */
@@ -172,13 +173,13 @@ readrecord(label)
  *	i)	label	record label
  *	i)	level	nest level for check
  */
-static	void
+static void
 includelabel(sb, label, level)
 	STRBUF	*sb;
 	const char *label;
 	int	level;
 {
-	char *savep, *p, *q;
+	const char *savep, *p, *q;
 
 	if (++level > allowed_nest_level)
 		die("nested include= (or tc=) over flow.");
@@ -196,41 +197,43 @@ includelabel(sb, label, level)
 		strbuf_close(inc);
 	}
 	strbuf_puts(sb, p);
-	free(savep);
+	free((void *)savep);
 }
 /*
  * configpath: get path of configuration file.
  */
 static char *
-configpath() {
-	static char config[MAXPATHLEN+1];
-	char *p;
+configpath(void)
+{
+	STATIC_STRBUF(sb);
+	const char *p;
 
+	strbuf_clear(sb);
 	/*
 	 * at first, check environment variable GTAGSCONF.
 	 */
 	if (getenv("GTAGSCONF") != NULL)
-		strlimcpy(config, getenv("GTAGSCONF"), sizeof(config));
+		strbuf_puts(sb, getenv("GTAGSCONF"));
 	/*
 	 * if GTAGSCONF not set then check standard config files.
 	 */
 	else if ((p = get_home_directory()) && test("r", makepath(p, GTAGSRC, NULL)))
-		strlimcpy(config, makepath(p, GTAGSRC, NULL), sizeof(config));
+		strbuf_puts(sb, makepath(p, GTAGSRC, NULL));
 #ifdef __DJGPP__
 	else if ((p = get_home_directory()) && test("r", makepath(p, DOS_GTAGSRC, NULL)))
-		strlimcpy(config, makepath(p, DOS_GTAGSRC, NULL), sizeof(config));
+		strbuf_puts(sb, makepath(p, DOS_GTAGSRC, NULL));
 #endif
 	else if (test("r", GTAGSCONF))
-		strlimcpy(config, GTAGSCONF, sizeof(config));
+		strbuf_puts(sb, GTAGSCONF);
 	else if (test("r", OLD_GTAGSCONF))
-		strlimcpy(config, OLD_GTAGSCONF, sizeof(config));
+		strbuf_puts(sb, OLD_GTAGSCONF);
 	else if (test("r", DEBIANCONF))
-		strlimcpy(config, DEBIANCONF, sizeof(config));
+		strbuf_puts(sb, DEBIANCONF);
 	else if (test("r", OLD_DEBIANCONF))
-		strlimcpy(config, OLD_DEBIANCONF, sizeof(config));
+		strbuf_puts(sb, OLD_DEBIANCONF);
 	else
 		return NULL;
-	return config;
+	return strbuf_value(sb);
 }
 /*
  * openconf: load configuration file.
@@ -238,10 +241,10 @@ configpath() {
  *	go)	line	specified entry
  */
 void
-openconf()
+openconf(void)
 {
 	STRBUF *sb;
-	char *config;
+	const char *config;
 	extern int vflag;
 
 	assert(opened == 0);
@@ -302,9 +305,22 @@ openconf()
 	sb = strbuf_open(0);
 	strbuf_puts(sb, confline);
 	strbuf_unputc(sb, ':');
+
 	if (!getconfs("suffixes", NULL)) {
+		STRBUF *tmp = strbuf_open(0);
+		const char *langmap = NULL;
+
+		/*
+		 * Variable 'suffixes' is obsoleted. But it is generated
+		 * internally from the value of variable 'langmap'.
+		 */
+		if (getconfs("langmap", tmp))
+			langmap = strbuf_value(tmp);
+		else
+			langmap = DEFAULTLANGMAP;
 		strbuf_puts(sb, ":suffixes=");
-		strbuf_puts(sb, DEFAULTSUFFIXES);
+		make_suffixes(langmap, sb);
+		strbuf_close(tmp);
 	}
 	if (!getconfs("skip", NULL)) {
 		strbuf_puts(sb, ":skip=");
@@ -316,19 +332,19 @@ openconf()
 	 * (Otherwise, nothing to do for gtags.)
 	 */
 	if (!getconfs("GTAGS", NULL) && !getconfs("GRTAGS", NULL) && !getconfs("GSYMS", NULL)) {
-		char *path;
+		const char *path;
 
 		/*
-		 * Some GNU/Linux has gctags as '/usr/bin/gctags', that is
-		 * different from GLOBAL's one.
 		 * usable search in BINDIR at first.
 		 */
-#if defined(_WIN32) || defined(__DJGPP__)
-		path = "gctags.exe";
+#if defined(_WIN32)
+		path = "gtags-parser.exe";
+#elif defined(__DJGPP__)
+		path = usable("gtags-parser") ? "gtags-parser.exe" : "gtags-~1.exe";
 #else
-		path = usable("gctags");
+		path = usable("gtags-parser");
 		if (!path)
-			path = "gctags";
+			path = "gtags-parser";
 #endif /* _WIN32 */
 		strbuf_puts(sb, ":GTAGS=");
 		strbuf_puts(sb, path);
@@ -361,7 +377,7 @@ getconfn(name, num)
 	const char *name;
 	int *num;
 {
-	char *p;
+	const char *p;
 	char buf[MAXPROPLEN+1];
 
 	if (!opened)
@@ -387,7 +403,7 @@ getconfs(name, sb)
 	const char *name;
 	STRBUF *sb;
 {
-	char *p;
+	const char *p;
 	char buf[MAXPROPLEN+1];
 	int all = 0;
 	int exist = 0;
@@ -448,15 +464,15 @@ getconfb(name)
 /*
  * getconfline: print loaded config entry.
  */
-char *
-getconfline()
+const char *
+getconfline(void)
 {
 	if (!opened)
 		openconf();
 	return confline;
 }
 void
-closeconf()
+closeconf(void)
 {
 	if (!opened)
 		return;
