@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001 Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifndef _DBOP_H_
@@ -61,13 +61,13 @@ typedef	struct {
 #define DBOP_PREFIX	2		/* prefixed read		*/
 
 DBOP *dbop_open(const char *, int, int, int);
-char *dbop_get(DBOP *, const char *);
+const char *dbop_get(DBOP *, const char *);
 void dbop_put(DBOP *, const char *, const char *);
 void dbop_delete(DBOP *, const char *);
 void dbop_update(DBOP *, const char *, const char *);
-char *dbop_first(DBOP *, const char *, regex_t *, int);
-char *dbop_next(DBOP *);
-char *dbop_lastdat(DBOP *);
+const char *dbop_first(DBOP *, const char *, regex_t *, int);
+const char *dbop_next(DBOP *);
+const char *dbop_lastdat(DBOP *);
 void dbop_close(DBOP *);
 
 #endif /* _DBOP_H_ */
