@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2002 Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000, 2002
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,7 +55,7 @@ static void
 setupvariables(verbose)
 	int verbose;
 {
-	char *p;
+	const char *p;
 
 	if ((p = getenv("MAKEOBJDIRPREFIX")) != NULL) {
 		makeobjdirprefix = p;
@@ -87,7 +87,7 @@ setupvariables(verbose)
  */
 int
 gtagsexist(candidate, dbpath, size, verbose)
-	char *candidate;
+	const char *candidate;
 	char *dbpath;
 	int size;
 	int verbose;
@@ -214,7 +214,7 @@ getdbpath(cwd, root, dbpath, verbose)
 		do {
 			FILE *fp;
 			STRBUF *sb;
-			char *s, *path;
+			const char *s, *path;
 
 			path = makepath(root, "GTAGSROOT", NULL);
 			if (!test("fr", path)) {
