@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000 Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,13 +16,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifndef _GETDBPATH_H_
 #define _GETDBPATH_H_
 
-int gtagsexist(char *, char *, int, int);
+int gtagsexist(const char *, char *, int, int);
 void getdbpath(char *, char *, char *, int);
 
 #endif /* ! _GETDBPATH_H_ */
