@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1997, 1998, 1999, 2000
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -18,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifndef _PATHOP_H_
@@ -30,15 +28,15 @@
 
 #define NEXTKEY		" __.NEXTKEY"
 
-int	gpath_open(const char *, int, int);
-char	*gpath_path2fid(const char *);
-char	*gpath_fid2path(const char *);
-void	gpath_put(const char *);
-void	gpath_delete(const char *);
-void	gpath_close(void);
-int	gpath_nextkey(void);
-void	gfind_open(char *, char *);
-char	*gfind_read(void);
-void	gfind_close(void);
+int gpath_open(const char *, int, int);
+const char *gpath_path2fid(const char *);
+const char *gpath_fid2path(const char *);
+void gpath_put(const char *);
+void gpath_delete(const char *);
+void gpath_close(void);
+int gpath_nextkey(void);
+void gfind_open(const char *, const char *);
+const char *gfind_read(void);
+void gfind_close(void);
 
 #endif /* ! _PATHOP_H_ */
