@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,9 +53,9 @@
 #include "strmake.h"
 #include "tab.h"
 
-static char	*unpack_pathindex(char *);
-static char	*genrecord(GTOP *);
-static int	belongto(GTOP *, char *, char *);
+static const char *unpack_pathindex(const char *);
+static const char *genrecord(GTOP *);
+static int belongto(GTOP *, const char *, const char *);
 static regex_t reg;
 
 /*
@@ -101,7 +101,6 @@ static regex_t reg;
  */
 static int support_version = 3;	/* acceptable format version   */
 static const char *tagslist[] = {"GPATH", "GTAGS", "GRTAGS", "GSYMS"};
-static STRBUF *output;
 /*
  * dbname: return db name
  *
@@ -129,11 +128,11 @@ dbname(db)
  */
 void
 makecommand(comline, path, sb)
-	char *comline;
-	char *path;
+	const char *comline;
+	const char *path;
 	STRBUF *sb;
 {
-	char *p = locatestring(comline, "%s", MATCH_FIRST);
+	const char *p = locatestring(comline, "%s", MATCH_FIRST);
 
 	if (p) {
 		strbuf_nputs(sb, comline, p - comline);
@@ -168,17 +167,17 @@ makecommand(comline, path, sb)
  */
 void
 formatcheck(line, format)
-	char *line;
+	const char *line;		/* virtually const */
 	int format;
 {
 	int n;
-	char *p;
+	const char *p;
 	SPLIT ptable;
 
 	/*
 	 * Extract parts.
 	 */
-	n = split(line, 4, &ptable);
+	n = split((char *)line, 4, &ptable);
 
 	/*
 	 * line number
@@ -230,8 +229,8 @@ formatcheck(line, format)
  */
 GTOP *
 gtags_open(dbpath, root, db, mode, flags)
-	char *dbpath;
-	char *root;
+	const char *dbpath;
+	const char *root;
 	int db;
 	int mode;
 	int flags;
@@ -307,7 +306,7 @@ gtags_open(dbpath, root, db, mode, flags)
 		 * if 'format version record' is not found, it's assumed
 		 * version 1.
 		 */
-		char *p;
+		const char *p;
 
 		if ((p = dbop_get(gtop->dbop, VERSIONKEY)) != NULL) {
 			for (p += strlen(VERSIONKEY); *p && isspace(*p); p++)
@@ -356,10 +355,10 @@ gtags_open(dbpath, root, db, mode, flags)
 void
 gtags_put(gtop, tag, record)
 	GTOP *gtop;
-	char *tag;
-	char *record;
+	const char *tag;
+	const char *record;		/* virtually const */
 {
-	char *line, *path;
+	const char *line, *path;
 	SPLIT ptable;
 
 	if (gtop->format == GTAGS_STANDARD || gtop->format == GTAGS_PATHINDEX) {
@@ -370,7 +369,7 @@ gtags_put(gtop, tag, record)
 	/*
 	 * gtop->format & GTAGS_COMPACT
 	 */
-	if (split(record, 4, &ptable) != 4) {
+	if (split((char *)record, 4, &ptable) != 4) {
 		recover(&ptable);
 		die("illegal tag format.\n'%s'", record);
 	}
@@ -413,15 +412,15 @@ gtags_put(gtop, tag, record)
 void
 gtags_add(gtop, comline, path, flags)
 	GTOP *gtop;
-	char *comline;
-	char *path;
+	const char *comline;
+	const char *path;
 	int flags;
 {
-	char *ctags_x;
+	const char *ctags_x;
 	FILE *ip;
 	STRBUF *sb = strbuf_open(0);
 	STRBUF *ib = strbuf_open(MAXBUFLEN);
-	char *fid;
+	const char *fid;
 
 	/*
 	 * add path index if not yet.
@@ -504,17 +503,17 @@ gtags_add(gtop, comline, path, flags)
 static int
 belongto(gtop, path, line)
 	GTOP *gtop;
-	char *path;
-	char *line;
+	const char *path;
+	const char *line;		/* virtually const */
 {
-	char *p = NULL;
+	const char *p = NULL;
 	int status, n;
 	SPLIT ptable;
 
 	/*
 	 * Get path.
 	 */
-	n = split(line, 4, &ptable);
+	n = split((char *)line, 4, &ptable);
 	if (gtop->format == GTAGS_STANDARD || gtop->format == GTAGS_PATHINDEX) {
 		if (n < 4) {
 			recover(&ptable);
@@ -539,9 +538,9 @@ belongto(gtop, path, line)
 void
 gtags_delete(gtop, path)
 	GTOP *gtop;
-	char *path;
+	const char *path;
 {
-	char *p;
+	const char *p;
 	/*
 	 * In compact format, a path is saved as a file number.
 	 */
@@ -573,17 +572,16 @@ gtags_delete(gtop, path)
  *			GTOP_BASICREGEX	use basic regular expression.
  *	r)		record
  */
-char *
+const char *
 gtags_first(gtop, pattern, flags)
 	GTOP *gtop;
-	char *pattern;
+	const char *pattern;
 	int flags;
 {
 	int dbflags = 0;
-	char *line;
-	char prefix[IDENTLEN+1], *p;
+	char prefix[IDENTLEN+1];
 	regex_t *preg = &reg;
-	char *key;
+	const char *key, *p, *line;
 	int regflags = 0;
 
 	gtop->flags = flags;
@@ -628,7 +626,7 @@ gtags_first(gtop, pattern, flags)
 	/*
 	 * Compact format.
 	 */
-	gtop->line = line;			/* gtop->line = $0 */
+	gtop->line = (char *)line;		/* gtop->line = $0 */
 	gtop->opened = 0;
 	return genrecord(gtop);
 }
@@ -639,11 +637,11 @@ gtags_first(gtop, pattern, flags)
  *	r)		record
  *			NULL end of tag
  */
-char *
+const char *
 gtags_next(gtop)
 	GTOP *gtop;
 {
-	char *line;
+	const char *line;
 	/*
 	 * If it is standard format or only key.
 	 * Just return it.
@@ -669,7 +667,7 @@ gtags_next(gtop)
 	 */
 	if ((line = dbop_next(gtop->dbop)) == NULL)
 		return line;
-	gtop->line = line;			/* gtop->line = $0 */
+	gtop->line = (char *)line;		/* gtop->line = $0 */
 	gtop->opened = 0;
 	return genrecord(gtop);
 }
@@ -698,15 +696,16 @@ gtags_close(gtop)
  *
  *	i)	line	tag line
  */
-static char *
+static const char *
 unpack_pathindex(line)
-	char *line;
+	const char *line;		/* virtually const */
 {
+	STATIC_STRBUF(output);
 	SPLIT ptable;
 	int n;
-	char *path;
+	const char *path;
 
-	n = split(line, 4, &ptable);
+	n = split((char *)line, 4, &ptable);
 	if (n < 4) {
 		recover(&ptable);
 		die("illegal tag format.'%s'\n", line);
@@ -721,10 +720,7 @@ unpack_pathindex(line)
 	/*
 	 * copy line with converting.
 	 */
-	if (output == NULL)
-		output = strbuf_open(MAXBUFLEN);
-	else
-		strbuf_reset(output);
+	strbuf_clear(output);
 	strbuf_nputs(output, line, ptable.part[2].start - line);
 	strbuf_puts(output, path);
 	strbuf_puts(output, ptable.part[2].end);
@@ -737,7 +733,7 @@ unpack_pathindex(line)
  *	io)	gtop	GTOP structure
  *	r)		tag line
  */
-static char *
+static const char *
 genrecord(gtop)
 	GTOP *gtop;
 {
@@ -745,13 +741,13 @@ genrecord(gtop)
 	static char output[MAXBUFLEN+1];
 	char path[MAXPATHLEN+1];
 	static char buf[1];
-	char *buffer = buf;
-	char *lnop;
+	const char *buffer = buf;
+	const char *lnop;
 	int tagline;
 
 	if (!gtop->opened) {
 		int n;
-		char *p;
+		const char *p;
 
 		gtop->opened = 1;
 		n = split(gtop->line, 3, &ptable);
@@ -769,7 +765,7 @@ genrecord(gtop)
 		 */
 		p = ptable.part[1].start;
 		if (gtop->format & GTAGS_PATHINDEX) {
-			char *q;
+			const char *q;
 			if ((q = gpath_fid2path(p)) == NULL)
 				die("GPATH is corrupted.('%s' not found)", p);
 			p = q;
