@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,11 +33,10 @@
 #include "makepath.h"
 #include "strbuf.h"
 
-static STRBUF	*sb;
 /*
  * makepath: make path from directory and file.
  *
- *	i)	dir	directory
+ *	i)	dir	directory(optional)
  *	i)	file	file
  *	i)	suffix	suffix(optional)
  *	r)		path
@@ -60,23 +59,24 @@ makepath(dir, file, suffix)
 	const char *file;
 	const char *suffix;
 {
+	STATIC_STRBUF(sb);
 	int length;
 	char sep = '/';
 
-	if (sb == NULL)
-		sb = strbuf_open(0);
-	strbuf_reset(sb);
-	if ((length = strlen(dir)) > MAXPATHLEN)
-		die("path name too long. '%s'\n", dir);
+	strbuf_clear(sb);
+	if (dir != NULL) {
+		if ((length = strlen(dir)) > MAXPATHLEN)
+			die("path name too long. '%s'\n", dir);
 
 #if defined(_WIN32) || defined(__DJGPP__)
-	/* follows native way. */
-	if (dir[0] == '\\' || dir[2] == '\\')
-		sep = '\\';
+		/* follows native way. */
+		if (dir[0] == '\\' || dir[2] == '\\')
+			sep = '\\';
 #endif
-	strbuf_puts(sb, dir);
-	strbuf_unputc(sb, sep);
-	strbuf_putc(sb, sep);
+		strbuf_puts(sb, dir);
+		strbuf_unputc(sb, sep);
+		strbuf_putc(sb, sep);
+	}
 	strbuf_puts(sb, file);
 	if (suffix) {
 		if (*suffix != '.')
