@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2002 Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2005
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -81,7 +81,7 @@ static void print_and_abort (void);
 void (*strbuf_alloc_failed_handler) (void) = print_and_abort;
 
 static void
-print_and_abort()
+print_and_abort(void)
 {
 	die("short of memory.");
 }
@@ -93,7 +93,7 @@ STRBUF	top;
  */
 void
 strbuf_dump(msg)
-	char *msg;
+	const char *msg;
 {
 	STRBUF *sb;
 	int i = 0;
@@ -108,7 +108,7 @@ strbuf_dump(msg)
 		if (strbuf_getlen(sb) <= strlen(p))
 			fprintf(stderr, ", value=|%s|\n", p);
 		else {
-			fprintf(stderr, "\n");
+			fputc('\n', stderr);
 			for (; p < end; p += strlen(p) + 1)
 				fprintf(stderr, "\t|%s|", p);
 		}
@@ -183,6 +183,93 @@ strbuf_open(init)
 #endif
 	return sb;
 }
+/*
+ * strbuf_clear: clear static string buffer.
+ *
+ *	i)	sb	statically defined string buffer
+ *
+ * This function is used for the initializing of static string buffer.
+ * For the detail, see 'STATIC_STRBUF(sb)' macro in strbuf.h.
+ */
+void
+strbuf_clear(sb)
+	STRBUF *sb;
+{
+	if (sb == NULL)
+		die("NULL string buffer. (strbuf_clear)");
+	if (strbuf_empty(sb)) {
+		sb->sbufsize = INITIALSIZE;
+		if (!(sb->sbuf = (char *)malloc(sb->sbufsize + 1))) {
+			(*strbuf_alloc_failed_handler)();
+			return;
+		}
+		sb->curp = sb->sbuf;
+		sb->endp = sb->sbuf + sb->sbufsize;
+	} else {
+		strbuf_reset(sb);
+	}
+}
+/*
+ * strbuf_nputs: Put string with length
+ *
+ *	i)	sb	statically defined string buffer
+ *	i)	s	string
+ *	i)	len	length of string
+ */
+void
+strbuf_nputs(sb, s, len)
+	STRBUF *sb;
+	const char *s;
+	int len;
+{
+	if (!sb->alloc_failed && len > 0) {
+		if (sb->curp + len > sb->endp)
+			__strbuf_expandbuf(sb, len);
+		while (len-- > 0)
+			*sb->curp++ = *s++;
+	}
+}
+/*
+ * strbuf_puts: Put string
+ *
+ *	i)	sb	statically defined string buffer
+ *	i)	s	string
+ */
+void
+strbuf_puts(sb, s)
+	STRBUF *sb;
+	const char *s;
+{
+	if (!sb->alloc_failed) {
+		while (*s) {
+			if (sb->curp >= sb->endp)
+				__strbuf_expandbuf(sb, 0);
+			*sb->curp++ = *s++;
+		}
+	}
+}
+/*
+ * strbuf_puts_nl: Put string with a new line
+ *
+ *	i)	sb	statically defined string buffer
+ *	i)	s	string
+ */
+void
+strbuf_puts_nl(sb, s)
+	STRBUF *sb;
+	const char *s;
+{
+	if (!sb->alloc_failed) {
+		while (*s) {
+			if (sb->curp >= sb->endp)
+				__strbuf_expandbuf(sb, 0);
+			*sb->curp++ = *s++;
+		}
+		if (sb->curp >= sb->endp)
+			__strbuf_expandbuf(sb, 0);
+		*sb->curp++ = '\n';
+	}
+}
 /*
  * strbuf_putn: put digit string at the last of buffer.
  *
@@ -194,18 +281,18 @@ strbuf_putn(sb, n)
 	STRBUF *sb;
 	int n;
 {
-	char num[128];
-	int i = 0;
-
-	while (n) {
-		if (i >= sizeof(num))
-			die("Too big integer value.");
-		num[i++] = n % 10 + '0';
-		n = n / 10;
-	}
-	if (i == 0) {
+	if (n == 0) {
 		strbuf_putc(sb, '0');
 	} else {
+		char num[128];
+		int i = 0;
+
+		while (n) {
+			if (i >= sizeof(num))
+				die("Too big integer value.");
+			num[i++] = n % 10 + '0';
+			n = n / 10;
+		}
 		while (--i >= 0)
 			strbuf_putc(sb, num[i]);
 	}
@@ -312,9 +399,9 @@ strbuf_fgets(sb, ip, flags)
  * strbuf_sprintf: do sprintf into string buffer.
  *
  *	i)	sb	STRBUF structure
- *	i)	s	same with sprintf()
- *
- * Node: Generated string is limited to 1024 bytes.
+ *	i)	s	similar to sprintf()
+ *			Currently the following format is supported.
+ *			%s, %d, %<number>d, %<number>s
  */
 void
 #ifdef HAVE_STDARG_H
@@ -322,12 +409,10 @@ strbuf_sprintf(STRBUF *sb, const char *s, ...)
 #else
 strbuf_sprintf(sb, s, va_alist)
 	STRBUF *sb;
-	char *s;
-	va_dcl;
+	const char *s;
+	va_dcl
 #endif
 {
-	static char buf[1024];
-	int i;
 	va_list ap;
 
 #ifdef HAVE_STDARG_H
@@ -335,15 +420,63 @@ strbuf_sprintf(sb, s, va_alist)
 #else
 	va_start(ap);
 #endif
-	/* This should be replaced with vsnprintf() in the future. */
-	(void)vsprintf(buf, s, ap);
-	for (i = 0; i < sizeof(buf); i++)
-		if (buf[i] == '\0')
+	if (sb->alloc_failed)
+		return;
+	for (; *s; s++) {
+		/*
+		 * Put the before part of '%'.
+		 */
+		{
+			const char *p;
+			for (p = s; *p && *p != '%'; p++)
+				;
+			if (p > s) {
+				strbuf_nputs(sb, s, p - s);
+				s = p;
+			}
+		}
+		if (*s == '\0')
 			break;
-	if (i >= sizeof(buf))
-		die("internal buffer for strbuf_sprintf over flow.");
+		if (*s == '%') {
+			int c = *++s;
+			/*
+			 * '%%' means '%'.
+			 */
+			if (c == '%') {
+				strbuf_putc(sb, c);
+			}
+			/*
+			 * If the optional number is specified then
+			 * we forward the job to snprintf(3).
+			 * o %<number>d
+			 * o %<number>s
+			 */
+			else if (isdigit(c)) {
+				char format[32], buf[1024];
+				int i = 0;
+
+				format[i++] = '%';
+				while (isdigit(*s))
+					format[i++] = *s++;
+				format[i++] = c = *s;
+				format[i] = '\0';
+				if (c == 'd' || c == 'x')
+					snprintf(buf, sizeof(buf), format, va_arg(ap, int));
+				else if (c == 's')
+					snprintf(buf, sizeof(buf), format, va_arg(ap, char *));
+				else
+					die("Unsupported control character '%c'.", c);
+				strbuf_puts(sb, buf);
+			} else if (c == 's') {
+				strbuf_puts(sb, va_arg(ap, char *));
+			} else if (c == 'd') {
+				strbuf_putn(sb, va_arg(ap, int));
+			} else {
+				die("Unsupported control character '%c'.", c);
+			}
+		}
+	}
 	va_end(ap);
-	strbuf_puts(sb, buf);
 }
 /*
  * strbuf_close: close string buffer.
@@ -363,6 +496,36 @@ strbuf_close(sb)
 	(void)free(sb->sbuf);
 	(void)free(sb);
 }
+/*
+ * Temporary string buffer for general purpose.
+ *
+ * Usage:
+ *
+ *	STRBUF *sbt = strbuf_open_tempbuf();
+ *	....
+ *	strbuf_puts(sbtemp, "xxx");
+ *	...
+ *	strbuf_release_tempbuf(sbt);
+ *
+ */
+int used = 0;
+
+STRBUF *
+strbuf_open_tempbuf(void)
+{
+	STATIC_STRBUF(sb);
+	if (used)
+		die("Internal error: temporary string buffer is already used.");
+	used = 1;
+	strbuf_clear(sb);
+	return sb;
+}
+void
+strbuf_release_tempbuf(sb)
+	STRBUF *sb;
+{
+	used = 0;
+}
 #ifdef STRBUF_LINK
 /*
  * strbuf_setname: set name to specified string buffer.
@@ -373,7 +536,7 @@ strbuf_close(sb)
 void
 strbuf_setname(sb, name)
 	STRBUF  *sb;
-	char	*name;
+	const char *name;
 {
 	char *p = strdup(name);
 	if (p == NULL) {
@@ -393,7 +556,7 @@ strbuf_setname(sb, name)
  */
 STRBUF *
 strbuf_getbuf(name)
-	char	*name;
+	const char *name;
 {
 	STRBUF *sb;
 
@@ -406,7 +569,7 @@ strbuf_getbuf(name)
  * strbuf_closeall: close all string buffer.
  */
 void
-strbuf_closeall()
+strbuf_closeall(void)
 {
 	while (top.next != &top)
 		strbuf_close(top.next);
