@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000 Tama Communications Corporation
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -51,7 +51,7 @@ settabs(n)
 void
 detab(op, buf)
 	FILE *op;
-	char *buf;
+	const char *buf;
 {
 	int src, dst;
 	char c;
