@@ -16,7 +16,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #
 # Usage:
 #
@@ -27,7 +27,7 @@ case $1 in
 	exit 0;;
 esac
 prog='autoreconf flex gperf perl bison'	# required programs
-file='convert.pl configure.ac Makefile.am gctags/reserved.pl'	# required files
+file='convert.pl configure.ac Makefile.am gtags-parser/reserved.pl'	# required files
 
 echo "- File existent checking..."
 for f in `echo $file`; do
@@ -63,7 +63,7 @@ done
 # flex, bison and gperf.
 #
 echo "- Preparing parser source ..."
-(cd gctags; set -x
+(cd gtags-parser; set -x
 for lang in c cpp java php asm; do
 	name=${lang}_res
 	perl ./reserved.pl --prefix=$lang ${lang}_res.in > ${name}.gpf
@@ -87,7 +87,7 @@ done
 )
 
 echo "- Collecting reference manuals ..."
-commands="global gtags htags gctags gozilla";
+commands="global gtags htags gtags-parser gozilla";
 perl ./convert.pl --menu $commands > doc/reference.txi
 for d in `echo $commands`; do
 	perl ./convert.pl --info $d/manual.in > doc/$d.ref
@@ -110,6 +110,10 @@ case $1 in
 	./configure CFLAGS='-g -p -Wall -DDEBUG'
 	make -s
 	;;
+--warn)
+	./configure CFLAGS='-g -O2 -Wall'
+	make -s
+	;;
 -c|--configure|--make|--install)
 	./configure
 	;;
